@@ -801,8 +801,6 @@ swc_run_ssh_command (
 		int nhops,
 		int * o_fdar,
 		int * i_fdar,
-		char * t_pty_name,
-		size_t t_pty_name_size,
 		struct termios * login_orig_termiosP,
 		struct winsize * login_sizeP, 
 		pid_t * pump_pid,
@@ -909,8 +907,6 @@ swc_run_ssh_command (
 		 	i_fdm[1],
 			i_fdm[0],
 			i_fdm[2],
-			t_pty_name, 
-			t_pty_name_size, 
 			login_orig_termiosP, 
 			login_sizeP, pump_pid, make_master_raw, ignoremask);
 
