@@ -63,8 +63,6 @@
 		int nhops, 
 		int * o_fdar, 
 		int * i_fdar, 
-		char * t_pty_name, 
-		size_t t_pty_name_size, 
 		struct termios * login_orig_termiosP, 
 		struct winsize * login_sizeP,  
 		pid_t * pump_pid, 
