@@ -64,6 +64,18 @@ int get_stderr_fd(void);
 
 /** -------------------------------------- */
 
+static
+void
+init_header_root(ETAR * etar)
+{
+	etar_set_typeflag(etar, REGTYPE);
+	etar_set_mode_ul(etar, (unsigned int)(0440));
+	etar_set_uid(etar, 0);
+	etar_set_gid(etar, 0);
+	etar_set_uname(etar, "root");
+	etar_set_gname(etar, "root");
+}
+
 /*
  This purpose of the g_script_array[i] is
  to store the scripts in a form that can easily
@@ -221,9 +233,8 @@ audit_execution_scripts(SWI * swi, SCARY ** scary_scripts)
 	strob_close(tagspec);
 }
 
-static
 int
-write_trailer_blocks(int ofd, int nblocks)
+swinstall_lib_write_trailer_blocks(int ofd, int nblocks)
 {
 	int ret;
 	int retval = 0;
@@ -347,12 +358,7 @@ write_single_file_tar_archive(SWI * swi, int ofd, char * name, char * data)
 
 	etar = etar_open(swi->xformatM->taruM);
 	etar_init_hdr(etar);
-	etar_set_typeflag(etar, REGTYPE);
-	etar_set_mode_ul(etar, (unsigned int)(0440));
-	etar_set_uid(etar, 0);
-	etar_set_gid(etar, 0);
-	etar_set_uname(etar, "root");
-	etar_set_gname(etar, "root");
+	init_header_root(etar);
 	etar_set_size(etar, strlen(data));
 	if (etar_set_pathname(etar, name)) SWLIB_FATAL("name too long");
 	etar_set_chksum(etar);
@@ -422,7 +428,7 @@ load_single_file_tar_archive (
 	 */
 
 	strob_sprintf(tmp, 0,
-		"%s\n"
+		"dd 2>/dev/null | %s\n"
 		"sw_retval=$?\n"	/* This line is required */
 		"dd of=/dev/null 2>/dev/null\n"
 		,
@@ -1296,61 +1302,75 @@ write_out_signature_member(SWI * swi, struct tar_header * ptar_hdr,
 	unsigned long package_filesize;
 	char * sig;
 	int siglen;
-	STROB * tmp = strob_open(1);
+	STROB * tmp = strob_open(16);
+	ETAR * etar;
 	
 	ifd = xformat_get_ifd(xformat);
+	etar = etar_open(swi->xformatM->taruM);
+	etar_init_hdr(etar);
 
 	/*
-	 * This reads the tar header.
+	 * Set the basic tar header fields
+	 */
+	init_header_root(etar);
+
+	/*
+	 * This reads the signature tar header from the package.
 	 */
 	uxfio_read(ifd, (void*)(ptar_hdr), TARRECORDSIZE);
 
+	/*
+	 * Determine the filesize of the signature file
+	 * It damn well should be 512 or 1024
+	 */
 	taru_otoul(ptar_hdr->size, &filesize);
-
 	package_filesize = filesize + TARRECORDSIZE;
 	*package_ret = package_filesize;
 
 	/*
 	 * Do a sanity check on the size.
 	 */
-
 	if (filesize != 512 && filesize != 1024) {
 		SWI_internal_fatal_error();
 	}
 
+	/*
+	 * allocate some temporary memory for this
+	 */
 	sig = malloc((size_t)filesize);
 	if (!sig) SWI_internal_fatal_error();
 
 	/*
 	 * This reads the sigfile archive member data
 	 */	
-
 	uxfio_read(ifd, (void*)(sig), (size_t)filesize);
-
 	sig[filesize-1] = '\0';
 	siglen = strlen(sig);
-	
-	OFF_TO_CHARS((int)siglen, ptar_hdr->size, 0 /*termch*/);
-	MODE_TO_CHARS(0640, ptar_hdr->mode, 0 /*termch*/);
-	/* strob_set_length(xformat->taruM->headerM, 512); */
-	memcpy((void*)(strob_str(xformat->taruM->headerM)), (void*)(ptar_hdr), 512);
 
-	xformat->taruM->header_lengthM = TARRECORDSIZE;
+	/*
+	 * The package contains a sigfile padded with NULs.
+	 * We will only install the ascii bytes which means the
+	 * the sig file in the installed catalog will be shorter in
+	 * length than in the package.
+	 */
+
+	etar_set_size(etar, siglen);
+	etar_set_mode_ul(etar, (unsigned int)(0640));
+
 	strob_strcpy(tmp, SWINSTALL_INCAT_NAME);
 	swlib_unix_dircat(tmp, SWINSTALL_CATALOG_TAR);
 	swlib_squash_all_leading_slash(strob_str(tmp));  /* FIXME */
-	if (signum <= 1)
+	if (signum <= 1) {
 		strob_sprintf(tmp, 1, ".sig");
-	else
+	} else {
 		strob_sprintf(tmp, 1, ".sig%d", signum);
-
-	if (taru_set_new_name(xformat->taruM, NULL, 0, strob_str(tmp))) {
-		SWI_internal_fatal_error();
 	}
-	memcpy((void*)(ptar_hdr), (void*)(strob_str(xformat->taruM->headerM)), 512);
+
+	if (etar_set_pathname(etar, strob_str(tmp))) SWLIB_FATAL("name too long");
+	etar_set_chksum(etar);
 
 	if (opt_preview == 0) {
-		ret = write(ofd, (void*)(ptar_hdr), TARRECORDSIZE);
+		ret = write(ofd, (void*)(etar_get_hdr(etar)), TARRECORDSIZE);
 		if (ret != TARRECORDSIZE) {
 			SWI_internal_fatal_error();
 		}
@@ -1372,6 +1392,7 @@ write_out_signature_member(SWI * swi, struct tar_header * ptar_hdr,
 		;
 	}
 
+	etar_close(etar);
 	strob_close(tmp);
 	free(sig);
 	return retval;
@@ -1532,7 +1553,7 @@ write_catalog_data(SWI * swi, int ofd, int sig_block_start, int sig_block_end)
 	/*
 	 * Now write 2 NUL trailer blocks
 	 */
-	ret = write_trailer_blocks(ofd, 2);
+	ret = swinstall_lib_write_trailer_blocks(ofd, 2);
 	if (ret < 0) return ret;
 	retval += ret;
 	return retval;
@@ -1550,28 +1571,36 @@ write_catalog_archive_member(SWI * swi, int ofd,
 	char * vendor_tag;
 	int size;
 	struct tar_header * catalog_tar_hdr;
+	ETAR * etar;
 
 	catalog_tar_hdr = (struct tar_header *)malloc(TARRECORDSIZE);
 	xformat = swi->xformatM;
 	buf = strob_open(100);
+	etar = etar_open(swi->xformatM->taruM);
+	etar_init_hdr(etar);
 
-	memcpy(catalog_tar_hdr, (void*)(strob_str(xformat->taruM->headerM)), TARRECORDSIZE);
+	/*
+	 * Set the basic tar header fields
+	 */
+	init_header_root(etar);
+
+	/*
+	 * set the tar header name and file size
+	 */
 	strob_strcpy(buf, "");
 	swlib_unix_dircat(buf, SWINSTALL_INCAT_NAME);
 	swlib_unix_dircat(buf, SWINSTALL_CATALOG_TAR);
 	size = write_catalog_data(swi, swi->nullfdM, sig_block_start, sig_block_end);
-
-	OFF_TO_CHARS(size, (catalog_tar_hdr)->size, 0 /*termch*/);
-	ptar_hdr->typeflag = REGTYPE;
-	MODE_TO_CHARS(0440, (catalog_tar_hdr)->mode, 0 /*termch*/);
-	taru_set_new_name(xformat->taruM, catalog_tar_hdr, 512, strob_str(buf));
+	etar_set_size(etar, size);
+	if (etar_set_pathname(etar, strob_str(buf))) SWLIB_FATAL("name too long");
+	etar_set_chksum(etar);
 
 	/*
 	 * write the tar header
 	 */
 	ret = atomicio((ssize_t (*)(int, void *, size_t))uxfio_write,
 				ofd,
-				(void*)(catalog_tar_hdr),
+				(void*)(etar_get_hdr(etar)),
 				(size_t)(TARRECORDSIZE));
 
 	if (ret != TARRECORDSIZE) {
@@ -1595,28 +1624,29 @@ write_catalog_archive_member(SWI * swi, int ofd,
 	 */
 	vendor_tag = strar_get(swi->distdataM->vendor_tagsM, 0 /*product index*/);
 	if (vendor_tag && strlen(vendor_tag)) {
-		SWLIB_ASSERT(strlen(vendor_tag) < 64); /* Posix Limit */
-		strob_strcpy(buf, SW_A_vendor_tag);
-		OFF_TO_CHARS(strlen(vendor_tag)+1, (catalog_tar_hdr)->size, 0 /*termch*/);
-		taru_set_new_name(xformat->taruM, catalog_tar_hdr, 512, strob_str(buf));
-		MODE_TO_CHARS(0444, (catalog_tar_hdr)->mode, 0 /*termch*/);
-		taru_set_tar_header_sum(xformat->taruM, (catalog_tar_hdr));
+		SWLIB_ASSERT(strlen(vendor_tag) < 64); /* Sanity check and enforce the Posix Limit */
+		init_header_root(etar);
+		etar_set_size(etar, strlen(vendor_tag) + 1 /* +1 is the newline */);
+		if (etar_set_pathname(etar, SW_A_vendor_tag)) SWLIB_FATAL("name too long");
+		etar_set_chksum(etar);
+
 		/*
 		 * write the tar header for the vendor_tag file.
+		 * The file name is "vendor_tag"
 		 */
 		ret = atomicio((ssize_t (*)(int, void *, size_t))uxfio_write,
 				ofd,
-				(void*)(catalog_tar_hdr),
+				(void*)(etar_get_hdr(etar)),
 				(size_t)(TARRECORDSIZE));
 		if (ret != TARRECORDSIZE) { SWBIS_ERROR_IMPL(); SWI_internal_fatal_error(); }
 		retval += ret;
 
 		/*
-		 * now write the data block.
+		 * now write the data block which the value of the vendor_tag.
 		 */
 		memset((void*)(swi->tarbufM), '\0', TARRECORDSIZE);
 		strncpy((char*)(swi->tarbufM), vendor_tag, TARRECORDSIZE - 2);
-		strcat((char*)(swi->tarbufM), "\n");
+		strcat((char*)(swi->tarbufM), "\n" /* FIXME \n must be 1 byte */);
 		ret = atomicio((ssize_t (*)(int, void *, size_t))uxfio_write,
 				ofd, (void*)(swi->tarbufM), (size_t)(TARRECORDSIZE));
 		if (ret != TARRECORDSIZE) { SWBIS_ERROR_IMPL(); SWI_internal_fatal_error(); }
@@ -1626,9 +1656,10 @@ write_catalog_archive_member(SWI * swi, int ofd,
 		 * Since their is no vendor_tag
 		 * write zero'ed tar blocks
 		 */
-		ret = write_trailer_blocks(ofd, 2);
+		ret = swinstall_lib_write_trailer_blocks(ofd, 2);
 		retval += ret;
 	}
+	etar_close(etar);
 	strob_close(buf);
 	free(catalog_tar_hdr);
 	return retval;
@@ -1680,7 +1711,7 @@ send_nothing_and_wait(SWI * swi, int ofd, int event_fd, char * msgtag, int tl, i
 	/* 
 	 * Send the payload
 	 */
-	ret = write_trailer_blocks(ofd, 1);
+	ret = swinstall_lib_write_trailer_blocks(ofd, 1);
 
 	/*
 	 * Reap the events from the event pipe.
@@ -1744,7 +1775,7 @@ send_signature_files(SWI * swi, int ofd,
 	 * Here is the task scriptlet.
 	 */
 	strob_sprintf(tmp, 0,
-		"%s\n"
+		"dd 2>/dev/null | %s\n"
 		"sw_retval=$?\n"
 		"dd of=/dev/null 2>/dev/null\n"
 		,
@@ -1869,7 +1900,7 @@ common_catalog_tarfile_operation(SWI * swi, int ofd,
 	 * This is needed because some dd(1) on some Un*xes can't
 	 * read zero blocks.
 	 */
-	ret = write_trailer_blocks(ofd, 1);
+	ret = swinstall_lib_write_trailer_blocks(ofd, 1);
 
 	if (ret < 0) {
 		SWBIS_ERROR_IMPL();
@@ -2115,7 +2146,7 @@ send_catalog_tarfile(SWI * swi, int ofd,
 	 * Here is the task scriptlet.
 	 */
 	strob_sprintf(tmp, 0,
-		"%s\n"
+		"dd 2>/dev/null | %s\n"
 		"sw_retval=$?\n"	/* This line is required */
 		"dd of=/dev/null 2>/dev/null\n"
 		,
@@ -2177,7 +2208,7 @@ send_catalog_tarfile(SWI * swi, int ofd,
 	/*
 	 * Now write the trailer blocks
 	 */	
-	ret = write_trailer_blocks(ofd, 2);
+	ret = swinstall_lib_write_trailer_blocks(ofd, 2);
 	if (ret < 0) {
 		SWBIS_ERROR_IMPL();
 		SWI_internal_error();
@@ -2248,7 +2279,7 @@ run_analysis_script(SWI * swi, int ofd, int event_fd)
 	/* 
 	 * Send the payload, in this case, it is a gratuitous block of nuls.
 	 */
-	ret = write_trailer_blocks(ofd, 1);
+	ret = swinstall_lib_write_trailer_blocks(ofd, 1);
 
 	/*
 	 * wait for the script to finish
@@ -2330,7 +2361,7 @@ swinstall_get_utsname_attributes(SWI * swi, int ofd, int event_fd)
 		return -1;
 	}
 	
-	ret = write_trailer_blocks(ofd, 1);
+	ret = swinstall_lib_write_trailer_blocks(ofd, 1);
 
 	ret = swicol_rpsh_task_expect(swi->swicolM,
 			event_fd,
@@ -2631,7 +2662,7 @@ arfinstall_tar(SWI * swi, int ofd, int efd, int do_preview,
 		}
 
 		/* Send the payload */
-		ret = write_trailer_blocks(ofd, 1);
+		ret = swinstall_lib_write_trailer_blocks(ofd, 1);
 
 		ret = swicol_rpsh_task_expect(swi->swicolM,	
 					event_fd,
@@ -3157,8 +3188,14 @@ arfinstall_tar(SWI * swi, int ofd, int efd, int do_preview,
 				} else {
 					/*
 					 * skip this file because its volatile and policy excludes it.
+					 * we must move the file pointer, just dump the data to * /dev/null
 					 */		
-
+					ret = xformat_copy_pass_md5(xformat,
+						swi->nullfdM,
+						ifd, md5buf);
+					SWLIB_ASSERT(ret >= 0);
+					optacc_a = 0;
+					continue;
 				}
 			} else {
 				/*
@@ -3254,15 +3291,17 @@ arfinstall_tar(SWI * swi, int ofd, int efd, int do_preview,
 				 */
 				header_ret = xformat_write_header(xformat);
 			}	
-
+	
 			if (header_ret < 0) {
 				SWI_internal_error();
 				return -76;
 			}	
 			fileset_write_ret += header_ret;
-				
+			
 			if (*current_type == SW_ITYPE_f) {
-				ret = xformat_copy_pass_md5(xformat, fileset_fd /* swi->nullfdM */, ifd, md5buf);
+				ret = xformat_copy_pass_md5(xformat,
+						fileset_fd,
+						ifd, md5buf);
 			} else {
 				ret = 0;
 			}
@@ -3280,7 +3319,7 @@ arfinstall_tar(SWI * swi, int ofd, int efd, int do_preview,
 					 * Sanity check
 					 */
 
-					if (ret < info_ahs2->file_hdrM->c_filesize) {
+					if (ret < (int)(info_ahs2->file_hdrM->c_filesize)) {
 						SWBIS_ERROR_IMPL();
 					}
 				}
@@ -3603,7 +3642,7 @@ write_target_install_script(
 	swlib_unix_dircat(catbuffer, distdata->catalog_bundle_dir1M);
 
 	/*
-	 * catbuffer contensts:
+	 * catbuffer contents:
 	 * <installed_software_catalog>/<bundle_tag>
 	 */
 
