@@ -198,6 +198,8 @@ class swExCat: public swObjFiles_i
 
 		next_line = swheader_get_next_object(swheader, (int)UCHAR_MAX, (int)UCHAR_MAX);
 		while (next_line){
+			swlib_doif_writef(swlib_get_verbose_level(), SWPACKAGE_VERBOSE_V3, NULL, swfdio_get(STDERR_FILENO),
+				"Export_step1: next_line=[%s]\n", next_line);
 			previous_swmd = swmd;
 			swmd = psf->swdeffile_linki_find_by_parserline(NULL, next_line);
 			SWLIB_ASSERT(swmd != NULL);
@@ -227,6 +229,11 @@ class swExCat: public swObjFiles_i
 					last_inext = last_def;
 					inext = last_def->get_next();
 					while(inext) {
+						if (swlib_get_verbose_level() >= SWPACKAGE_VERBOSE_V3) {
+							inext->write_fd_debug(
+								swfdio_get(STDERR_FILENO),
+								"Export_step1.1:");
+						}
 						if (inext == uplink) break;
 						last_inext = inext;
 						inext = inext->get_next();
@@ -266,6 +273,11 @@ class swExCat: public swObjFiles_i
 					last_inext = last_def;
 					inext = last_def->get_next();
 					while(inext) {
+						if (swlib_get_verbose_level() >= SWPACKAGE_VERBOSE_V3) {
+							inext->write_fd_debug(
+								swfdio_get(STDERR_FILENO),
+								"Export_step1.2:");
+						}
 						if (inext == uplink) break;
 						last_inext = inext;
 						inext = inext->get_next();
@@ -329,11 +341,21 @@ class swExCat: public swObjFiles_i
 
 		swexdist = static_cast<swExCat*>(swExFactory(current_line));  // Warning : Downcast !!
 		swexdist->setPSF(psf);
+		
+		swlib_doif_writef(swlib_get_verbose_level(), SWPACKAGE_VERBOSE_V3, NULL, swexdist->getStderrFd(),
+				"running constructSwExDist()\n");
 
 		//
 		// Need to initialize the link-lists in the <swDefinitionFile*> object.
 		//
+
+		swlib_doif_writef(swlib_get_verbose_level(), SWPACKAGE_VERBOSE_V3, NULL, swexdist->getStderrFd(),
+				"initializing: running psf->swdeffile_linki_init()\n");
+
 		psf->swdeffile_linki_init();
+		
+		swlib_doif_writef(swlib_get_verbose_level(), SWPACKAGE_VERBOSE_V3, NULL, swexdist->getStderrFd(),
+				"initializing: running psf->swdeffile_linki_init() Done\n");
 	
 		//
 		// Contruct the iterator and search object.
@@ -350,7 +372,11 @@ class swExCat: public swObjFiles_i
 		// Current_line should be the "distribution" or "host" object keyword.
 		//
 
+		swlib_doif_writef(swlib_get_verbose_level(), SWPACKAGE_VERBOSE_V3, NULL, swexdist->getStderrFd(),
+				"running psf->processForExport_step1()\n");
 		swexdist->processForExport_step1(swheader, psf, 0, current_line, error_code);
+		swlib_doif_writef(swlib_get_verbose_level(), SWPACKAGE_VERBOSE_V3, NULL, swexdist->getStderrFd(),
+				"running psf->processForExport_step1() Done\n");
 	
 		//
 		// Set the mem_fd back to its normal state.
@@ -370,6 +396,8 @@ class swExCat: public swObjFiles_i
 			//
 			swexdist->registerWithGlobalIndex();
 		}
+		swlib_doif_writef(swlib_get_verbose_level(), SWPACKAGE_VERBOSE_V3, NULL, swexdist->getStderrFd(),
+				"running constructSwExDist() Done\n");
 		return (swexdist);
 	}
 	
