@@ -255,7 +255,7 @@ class swExDistribution: public swExCat
 			case tuneSignatureFileE:
 			case tuneFilesFileE:
 			case setAllfilesetsE:
-			case lastE:
+			case last_taskE:
 					break;
 		}
 	}
@@ -285,7 +285,7 @@ class swExDistribution: public swExCat
 			case tuneSignatureFileE:
 			case tuneFilesFileE:
 			case setAllfilesetsE:
-			case lastE:
+			case last_taskE:
 			default:
 				swObjFiles_i::taskResponder(code);
 				break;
