@@ -175,6 +175,8 @@ class swExFileset: public swObjFiles_i
 		swDefinition * after_last_file = NULL;
 		swDefinition * last_def;
 
+		swlib_doif_writef(swlib_get_verbose_level(), SWPACKAGE_VERBOSE_V3, NULL, swfdio_get(STDERR_FILENO),
+			"swExFileset::Export_step1: ENTERING\n");
 
 		if (strlen(current_line) == 0) return NULL;
 
@@ -203,6 +205,8 @@ class swExFileset: public swObjFiles_i
 			*error_code = 0;
 		}
 		while (next_line){
+			swlib_doif_writef(swlib_get_verbose_level(), SWPACKAGE_VERBOSE_V3, NULL, swfdio_get(STDERR_FILENO),
+				"swExFileset::Export_step1: next_line=[%s]\n", next_line);
 			SWBIS_E_DEBUG2("next_line = [%s]", next_line);
 			previous_swmd = swmd;
 			swmd = psf->swdeffile_linki_find_by_parserline(NULL, next_line);
@@ -242,6 +246,8 @@ class swExFileset: public swObjFiles_i
 		}
 		SWBIS_E_DEBUG2("LEAVING --  after_last_file = [%p]", after_last_file);
 		SWBIS_E_DEBUG("LEAVING");
+		swlib_doif_writef(swlib_get_verbose_level(), SWPACKAGE_VERBOSE_V3, NULL, swfdio_get(STDERR_FILENO),
+			"swExFileset::Export_step1: LEAVING\n");
 		return after_last_file;
 	}
 	
@@ -336,7 +342,6 @@ class swExFileset: public swObjFiles_i
 		int do_adjunct = 0;
 		swPackageFile * archiver;
 		STROB * tmp;
-		STROB * buffer;
 
 		SWEXFILESET_DEBUG("");
 		ofd = get_ofd();
@@ -487,8 +492,8 @@ class swExFileset: public swObjFiles_i
 			
 			if (ret < 0) {
 				fprintf(stderr,
-					"error writing archive member for ( %s ) ret=%d\n",
-					strob_str(tmp), ret);
+					"%s: error writing archive member for ( %s ) ret=%d\n",
+					swlib_utilname_get(), strob_str(tmp), ret);
 				setErrorCode(12303);
 				return;
 			}
