@@ -83,6 +83,7 @@ class swExStruct_i: public swExStruct
 
   	swExStruct_i(void): swExStruct() { init(); }
 
+	int getStderrFd(void) { return STDERR_FILENO; }
 	int getNullFd(void) { return nullfdM; }
 
 	int get_ofd(void) { return outputfdM; }
@@ -554,7 +555,7 @@ class swExStruct_i: public swExStruct
 				//
 				setAllfilesets();
 				break;
-			case lastE:
+			case last_taskE:
 				break;
 			default:
 				break;
