@@ -318,6 +318,8 @@ class swObjFiles_i:  public swExStruct_i
 		swDefinition * first_file;
 		swDefinition * after_last_file;
 			
+		swlib_doif_writef(swlib_get_verbose_level(), SWPACKAGE_VERBOSE_V3, NULL, swfdio_get(STDERR_FILENO),
+			"swObjfiles_i::process_file_and_control_file_definitions: ENTERING\n");
 		info = new swINFO("");
 		SWLIB_ASSERT(info != NULL);
 		setInfo(info);
@@ -406,6 +408,8 @@ class swObjFiles_i:  public swExStruct_i
 			last_def->set_next(NULL);
 			first_file->set_prev(NULL);
 		}
+		swlib_doif_writef(swlib_get_verbose_level(), SWPACKAGE_VERBOSE_V3, NULL, swfdio_get(STDERR_FILENO),
+			"swObjfiles_i::process_file_and_control_file_definitions: LEAVING\n");
 		return after_last_file;
 	}
 
@@ -421,17 +425,21 @@ class swObjFiles_i:  public swExStruct_i
 		int did_find_duplicate;
 		char * next_line;
 		char * path;
+		char * tmpname;
 		swMetaData * previous_swmd = NULL;
 		swMetaData * swmd = NULL;
 		swMetaData * swat = NULL;
 		swDefinition * swmd_def = NULL;
 		CPLOB * filedeflist = cplob_open(10);
 		CPLOB * pathlist = cplob_open(10);
+		STROB * basenames = strob_open(100);
 			
 		//
 		// The first file definition was already read.
 		// So process it here.
 		//
+		swlib_doif_writef(swlib_get_verbose_level(), SWPACKAGE_VERBOSE_V3, NULL, swfdio_get(STDERR_FILENO),
+			"swObjfiles_i::process_files_step1: ENTERING\n");
 		SWLIB_ASSERT(first_file->get_type() == swstructdef::sdf_object_kw);
 		swat = first_file->findAttribute(SW_A_path);
 		if (swat) {
@@ -482,7 +490,7 @@ class swObjFiles_i:  public swExStruct_i
 
 			// fprintf(stderr, "eraseme swpackage: checking for duplicate here [%s]\n", path);
 			// swmd->write_fd(2);  // Heres howto write the definition to STDERR_FILENO
-			did_find_duplicate = check_for_storage_section_duplicates(filedeflist, pathlist, path);
+			did_find_duplicate = check_for_storage_section_duplicates(filedeflist, pathlist, path, basenames);
 
 			swmd_def = static_cast<swDefinition*>(swmd);
 			//
@@ -501,10 +509,25 @@ class swObjFiles_i:  public swExStruct_i
 				//
 				swmd_def->set_storage_status(SWDEF_STATUS_DUP);
 			}	
+
+			swlib_doif_writef(swlib_get_verbose_level(), SWPACKAGE_VERBOSE_V3, NULL, swfdio_get(STDERR_FILENO),
+				"swObjfiles_i::process_files_step1: next_line=[%s] path=%s\n", next_line, path ? path : "");
 	
 			if (path) {
 				cplob_add_nta(filedeflist, (char*)swmd_def);
 				cplob_add_nta(pathlist, (char*)path);
+				tmpname = ::strrchr(path, '/');
+				if (
+					tmpname && 
+					strlen(tmpname) > 1 &&
+					*(tmpname + 1) != '/' &&
+					*(tmpname + 1) != '.'
+				) {
+					strob_strcat(basenames, tmpname + 1);
+				} else {
+					strob_strcat(basenames, path);
+				}	
+				strob_strcat(basenames, "\n");
 			}
 
 			previous_swmd = swmd;
@@ -531,6 +554,9 @@ class swObjFiles_i:  public swExStruct_i
 		}
 		cplob_shallow_close(filedeflist);
 		cplob_shallow_close(pathlist);
+		strob_close(basenames);
+		swlib_doif_writef(swlib_get_verbose_level(), SWPACKAGE_VERBOSE_V3, NULL, swfdio_get(STDERR_FILENO),
+			"swObjfiles_i::process_files_step1: LEAVING\n");
 		return ret;
 	}
 
@@ -1207,7 +1233,6 @@ class swObjFiles_i:  public swExStruct_i
 	virtual void emitExportedCatalog(void) { 
 		swDefinitionFile * info;	
 		STROB * tmp = strob_open(100);
-		STROB * buffer = NULL;
 		swPtrList<swPackageFile> * archiveMemberList;
 		swPackageFile * swfile;
 		swPackageFile * nextswfile;
@@ -1289,6 +1314,7 @@ class swObjFiles_i:  public swExStruct_i
 			swfile->xFormat_set_perms(0755);
 			swfile->xFormat_set_user_systempair(getCatalogOwner());
 			swfile->xFormat_set_group_systempair(getCatalogGroup());
+			ret = 0;
 			if (is_duplicate == 0)
 				ret = swfile->xFormat_write_file();
 			if (ret < 0) setErrorCode(20008);
@@ -1334,7 +1360,10 @@ class swObjFiles_i:  public swExStruct_i
 	}
 
 	int
-	check_for_storage_section_duplicates(CPLOB * filedeflist, CPLOB * pathlist, char * incoming_path)
+	check_for_storage_section_duplicates(CPLOB * filedeflist,
+				CPLOB * pathlist,
+				char * incoming_path,
+				STROB * basenames)
 	{
 		int ret;
 		int retval = 0;
@@ -1343,6 +1372,8 @@ class swObjFiles_i:  public swExStruct_i
 		char ** current_path_p;
 		char ** path_array = cplob_get_list(pathlist);
 		swDefinition * duplicate_def;
+		char * incoming_base_name;
+		char * tmpname;
 
 		if (!incoming_path) return 0;
 
@@ -1370,6 +1401,34 @@ class swObjFiles_i:  public swExStruct_i
 		// Remember, the current swdef is not on the list being searched.
 		//
 
+
+		//
+		// check the basenames list.  This provides a much needed speed optimization.
+		//
+		incoming_base_name = NULL;
+		if ( 1 && 
+			(tmpname=strrchr(incoming_path, '/')) != static_cast<char*>(NULL) &&
+			strlen(tmpname) > 1 && // i.e. not just a trailing slash "/"
+			*(tmpname + 1) != '/' &&
+			*(tmpname + 1) != '.'
+		) {
+			incoming_base_name = tmpname + 1;
+		}
+
+		//
+		// Only attempt this optimization if we get a clean basename.
+		//
+		if (1 && 
+			incoming_base_name && *incoming_base_name != '/'
+		) {
+			if (strstr(strob_str(basenames), incoming_base_name) == static_cast<char*>(NULL)) {
+				//
+				// We can safely assume that the incoming name has no prior duplicate.
+				//
+				return 0;
+			}
+		}
+
 		current_path_p = path_array;
 		while(*current_path_p) {
 			ret = swlib_dir_compare(*current_path_p, incoming_path, SWC_FC_NOAB);
@@ -1441,7 +1500,6 @@ class swObjFiles_i:  public swExStruct_i
 		int old_fd;
 		int old_preview_fd;
 		int new_fd;
-		int new_preview_fd;
 		int datafd;
 		int data_len = 0;
 		int buffer_len = 0;
