@@ -4,7 +4,7 @@
  */
 
 /*
-   Copyright (C) 2003-2004 Jim Lowe
+   Copyright (C) 2003,2004,2005 Jim Lowe
    All Rights Reserved.
   
    COPYING TERMS AND CONDITIONS
@@ -103,6 +103,7 @@ class swExtendedDef : private swMetaData
 	int	do_prefix_distribution_targetM;
 	int	do_create_cksumM;
 	int	do_create_md5sumM;
+	int	non_existing_sourceM;
 
 	STROB * excludeM;
 	STROB * includeM;
@@ -159,7 +160,7 @@ public: // =====================================================================
 	int filePermissionsActive(void) { return filePermissionsActiveM; }
 	int directoryActive(void) { return directoryActiveM; }
 	int directoryDestActive(void) { return directoryDestActiveM; }
-			
+
 	// Public
 	//
 	void
@@ -805,6 +806,122 @@ private: // ================================================================
 		}
 	}
 
+	int
+	is_file_attribute_sufficient(int * do_no_stat, int is_set[], int is_default_set[])
+	{
+		int retval = 0;
+		//
+		// This routine implements Lines 810-812 Section 5.2.13.3
+		//
+		if (
+			(is_set[modeE] || (filePermissionsActiveM && is_default_set[modeE])) &&
+			(is_set[ownerE] || (filePermissionsActiveM && is_default_set[ownerE])) &&
+			(is_set[groupE] || (filePermissionsActiveM && is_default_set[groupE])) &&
+			1
+		) {
+			retval = 1;
+			if (do_no_stat) *do_no_stat = 1;
+		}
+		return retval;
+	}
+	
+	int
+	apply_default_values(int type)
+	{
+		int i;
+		int itmp;
+		mode_t mode;
+		unsigned int umask;
+		char * ptmp;
+		char  usernamebuf[TARU_SYSDBNAME_LEN+1];
+
+		//
+		// Apply default values in a way that is overridden by the
+		// file_permissions defaults if active.
+		//
+		umask = my_get_umask();
+		for (i=0; i<lastE; i++) {
+			//
+			// If default is active or explicitly set then
+			// do not apply a default.
+			//
+			// The defaults below are implementation defined (per spec)
+			// hence they are only meant to assign resonable values, not
+			// correct values (since correct is not defined).
+			//
+	
+			if (fileDefPermsArrayWasSetM[i])
+				continue;
+			if (fileArrayWasSetM[i])
+				continue;
+			switch(i) {
+				case typeE:
+					// assumed to be already set.
+					break;
+				case umaskE:
+					break;
+				case modeE:
+					mode = swlib_apply_mode_umask(type, umask, (mode_t)(0));
+					ptmp = get_file_cell_pointer(modeE);
+					snprintf(ptmp, file_permsLength-1, "%o", (unsigned int)mode);
+					ptmp[file_permsLength - 1] = '\0';
+					break;
+				case uidE:
+					ptmp = get_file_cell_pointer(uidE);
+					snprintf(ptmp, file_permsLength-1, "%d", (int)getuid());
+					ptmp[file_permsLength - 1] = '\0';
+					break;
+				case gidE:
+					ptmp = get_file_cell_pointer(gidE);
+					snprintf(ptmp, file_permsLength-1, "%d", (int)getgid());
+					ptmp[file_permsLength - 1] = '\0';
+					break;
+				case ownerE:
+					if (taru_get_user_by_uid(getuid(), usernamebuf) == 0) {
+						copy_file_cell(fileArrayM, fileArrayWasSetM, ownerE, usernamebuf);
+					} else {
+						//
+						// This should never happen
+						//
+						return -1;
+					}
+					break;
+				case groupE:
+					if (taru_get_group_by_gid(getgid(), usernamebuf) == 0) {
+						copy_file_cell(fileArrayM, fileArrayWasSetM, groupE, usernamebuf);
+					} else {
+						//
+						// This should never happen
+						//
+						return -1;
+					}
+					break;
+				case sizeE:
+				case volaE:
+				case compE:
+				case cksumE:
+				case md5sumE:
+				case mtimeE:
+				case ctimeE:
+				case inoE:
+				case nlinkE:
+				case majorE:
+				case minorE:
+				case linksourceE:
+				case swbiE:
+					*fileArrayM[i]='\0';
+					fileArrayWasSetM[i] = 0;
+					break;
+				default:
+					SWEXTDEF_DEBUG("");
+					fprintf(stderr,"swpackage: internal error in apply_default_values at i=%d.\n", i);
+					break;	
+			
+			}
+		}
+		return 0;
+	}
+
 	void
 	soft_init_(void) {
 		swdefFormedM = NULL;
@@ -818,6 +935,7 @@ private: // ================================================================
 		if (!getcwd(strob_str(cwdM), 250)) {
 			exit(3);
 		}
+		non_existing_sourceM = 0;
 	}
 	
 	void
@@ -826,6 +944,7 @@ private: // ================================================================
 		do_create_cksumM = 0;
 		do_create_md5sumM = 0;
 		did_open_swvarfsM = 0;
+		non_existing_sourceM = 0;
 	}
 
 	void
@@ -839,6 +958,7 @@ private: // ================================================================
 		directory_asourceM = strob_open(2);
 		directory_sourceM = strob_open(2);
 		directory_pathM = strob_open(2);
+		non_existing_sourceM = 0;
 	}
 
 	void
@@ -885,22 +1005,22 @@ private: // ================================================================
 			statArrayWasSetM[i] = 0;
 		}
 	}
-	
+
 	void
 	copy_file_cell(char array[][file_permsLength], int is_set[], int cell, char * value) {
 		if (value) {
 			strncpy(array[cell], value, file_permsLength -1);
 			array[cell][file_permsLength -1] = '\0';
-			is_set[cell] = 1;
+			if (is_set) is_set[cell] = 1;
 		} else {
-			is_set[cell] = 0;
+			if (is_set) is_set[cell] = 0;
 			strncpy(array[cell], "", file_permsLength -1);
 		}
 	}
 	
 	mode_t
 	my_get_umask(void) {
-		mode_t mode = 0777;
+		mode_t mode = 0;
 		mode = umask(mode);
 		umask(mode);
 		return mode;
@@ -935,6 +1055,9 @@ private: // ================================================================
 	local_access(char * name, int mode) {
 		int ret;
 		ret = local_lstat_swvarfs(name, &stM);
+		/*
+		 * FIXME use 'mode' to determine return value
+		 */
 		return ret;
 	}
 	
@@ -1562,8 +1685,6 @@ private: // ================================================================
 				int * p_skip_amount
 			)
 	{
-		int ret = -1;
-	
 		SWEXTDEF_DEBUG("");
 		
 		copy_file_cell(array, array_is_set, idix, NULL);
@@ -1880,10 +2001,13 @@ private: // ================================================================
 		mode_t default_umask = 022;
 		struct stat stbuf;
 		struct stat *st;
+		SWEXTDEF_DEBUG("");
 	
 		if (sti != static_cast<struct stat*>(NULL)) {
+			SWEXTDEF_DEBUG("");
 			st = sti;
 		} else {
+			SWEXTDEF_DEBUG("");
 			st = &stbuf;
 			if (source) {
 				if (local_lstat_swvarfs(source, st)) {
@@ -1897,9 +2021,14 @@ private: // ================================================================
 		}
 
 		if (source == static_cast<char*>(NULL)) {
+			//
+			// FIXME, this code branch probably should not exist here
+			//
+
 			//
 			// Set some default attribute values based on the type.
 			//
+			SWEXTDEF_DEBUG("");
 			p_mode = get_stat_cell_pointer(modeE);
 			p_minor = get_stat_cell_pointer(minorE);
 			p_major = get_stat_cell_pointer(majorE);
@@ -1924,13 +2053,15 @@ private: // ================================================================
 			snprintf(p_mode, file_permsLength-1, "%o", (unsigned int)mode);
 			p_mode[file_permsLength - 1] = '\0';
 			if (*p_owner == '\0') {
-				strncpy(p_owner, "daemon", 6);
+				SWEXTDEF_DEBUG("");
+				strncpy(p_owner, "daemon", 7);
 				strncpy(p_uid, "2", 6);
 				statArrayWasSetM[uidE] = 1;
 				statArrayWasSetM[ownerE] = 1;
 			}	
 			if (*p_group == '\0') {
-				strncpy(p_group, "daemon", 6);
+				SWEXTDEF_DEBUG("");
+				strncpy(p_group, "daemon", 7);
 				strncpy(p_gid, "2", 6);
 				statArrayWasSetM[gidE] = 1;
 				statArrayWasSetM[groupE] = 1;
@@ -1940,6 +2071,7 @@ private: // ================================================================
 		} else {
 			int cksumflags = 0;
 
+			SWEXTDEF_DEBUG("");
 			if (do_create_cksumM) {
 				cksumflags |= SWFILE_DO_CKSUM;
 			}
@@ -2015,12 +2147,8 @@ private: // ================================================================
 			is_set[name_index] = 0;
 		} else if (is_set[name_index] == 0 && is_set[id_index] == 0) {
 			SWEXTDEF_DEBUG("");
-			strncpy(array[name_index],  statarray[name_index], file_permsLength - 1);
-			array[name_index][file_permsLength - 1] = '\0';
-			strncpy(array[id_index],  statarray[id_index], file_permsLength - 1);
-			array[id_index][file_permsLength - 1] = '\0';
-			is_set[name_index] = 1;
-			is_set[id_index] = 1;
+			copy_file_cell(array, is_set, name_index, statarray[name_index]);
+			copy_file_cell(array, is_set, id_index, statarray[id_index]);
 		} else if (is_set[name_index] && is_set[id_index]) {
 			SWEXTDEF_DEBUG("");
 			;		
@@ -2062,9 +2190,7 @@ private: // ================================================================
 				//
 				if (file_cell_was_explicitly_set(i) == 0 && file_cell_was_default_set(i)) {
 					SWEXTDEF_DEBUG("");
-					strncpy(fileArrayM[i],  fileDefPermsArrayM[i], file_permsLength - 1);
-					fileArrayM[i][file_permsLength - 1] = '\0';
-					// fileArrayWasSetM[i] = 1;
+					copy_file_cell(fileArrayM, static_cast<int*>(NULL), i, fileDefPermsArrayM[i]);
 					set_file_was_set(fileArrayWasSetM, i);
 				}
 			}
@@ -2073,6 +2199,7 @@ private: // ================================================================
 		if (do_stat) {
 			SWEXTDEF_DEBUG("");
 			if (stat_source_file_if(source, hl_path, st)) {
+				fprintf(stderr,"swpackage: stat_source_file_if returned error.\n");
 				exit(2); // FIXME
 			}
 		}
@@ -2126,6 +2253,7 @@ private: // ================================================================
 		// the statArrayWasSetM[i] to zero.
 		//
 
+		SWEXTDEF_DEBUG("");
 		handle_ownerships(fileArrayM, fileArrayWasSetM, statArrayM, statArrayWasSetM, ownerE, uidE);
 		handle_ownerships(fileArrayM, fileArrayWasSetM, statArrayM, statArrayWasSetM, groupE, gidE);
 
@@ -2146,12 +2274,19 @@ private: // ================================================================
 						//
 						// Transfer the value from statArray to the fileArray.
 						//
+						SWEXTDEF_DEBUG("");
 						if (fileArrayWasSetM[i] == 0) {
-							strncpy(fileArrayM[i], statArrayM[i], file_permsLength - 1);
-							fileArrayM[i][file_permsLength - 1] = '\0';
-							// fileArrayWasSetM[i] = 1;
+							SWEXTDEF_DEBUG3("copying from stat array [%s] = [%s]", namesM[i], statArrayM[i]);
+							copy_file_cell(fileArrayM, static_cast<int*>(NULL), i, statArrayM[i]);
 							set_file_was_set(fileArrayWasSetM, i);
-							if (i == ctimeE) {
+							if (  0 ||
+								//
+								// These values are not in a stat buf or don't need preservation.
+								//
+								i == ctimeE || /* FIXME ?? why is this here */
+								i == umaskE ||
+								0
+							) {
 								fileArrayWasSetM[i] = 0;
 							}
 						}
@@ -2190,6 +2325,7 @@ private: // ================================================================
 		char type = 0;
 		unsigned int umask=0;
 
+		SWEXTDEF_DEBUG("");
 		for (cell=0; cell<lastE; cell++) {
 			value = get_file_cell_pointer(cell);
 
@@ -2220,16 +2356,55 @@ private: // ================================================================
 						SWEXTDEF_DEBUG("");
 						taru_otoul(value, &ulmode);
 						mode = (unsigned int)ulmode;
+						SWEXTDEF_DEBUG2(" mode value is [%o]", mode);
 					} else {
 						SWEXTDEF_DEBUG("");
 						mode = 0;
+						SWEXTDEF_DEBUG2(" mode value is [%o]", mode);
 					}
 
 					//
 					// type and umask are already set because of the ordering of the
 					// enumeration.
 					//
+					SWEXTDEF_DEBUG2("mode value before is [%o]", mode);
+					SWEXTDEF_DEBUG2("file type is [%c]", (int)type);
+					SWEXTDEF_DEBUG2("umask is [%o]", umask);
+
+					//
+					// Check here to see if umask was set explicitly, if not set
+					// then assign the Unix process value return by umask()
+					//
+					
+					if (file_cell_was_explicitly_set(modeE)) {
+						;
+						SWEXTDEF_DEBUG("mode already set");
+						// nothing to do
+					} else {
+						//
+						// Use the process umask
+						//
+						SWEXTDEF_DEBUG("setting umask with process value");
+
+						//
+						// This causes a regression test failure
+						// if used unconditionally
+						//
+						if (non_existing_sourceM) {
+							//
+							// Apply the process umask only for
+							// things like 'file -t d foobar'
+							// and where 'foobar' does not exist
+							//
+							umask = my_get_umask();
+						} else {
+
+						}
+					}
+					SWEXTDEF_DEBUG2("umask (now finally) is [%o]", umask);
+	
 					mode = swlib_apply_mode_umask(type, umask, mode);
+					SWEXTDEF_DEBUG2(" mode value after is [%o]", mode);
 					if (recursive_context && type == SW_ITYPE_d) {
 						//
 						// make sure the executable bit is on for any
@@ -2249,6 +2424,7 @@ private: // ================================================================
 					}
 					snprintf(value, file_permsLength, "%o", (unsigned int)mode);
 					value[file_permsLength - 1] = '\0';
+					SWEXTDEF_DEBUG2(" mode string value is [%s]", value);
 					break;
 				case uidE:
 				case gidE:
@@ -2465,32 +2641,99 @@ private: // ================================================================
 			//
 			SWEXTDEF_DEBUG("In type d");
 			if (is_path_true(o_source) && !is_path_true(o_path)) {
-				if (directoryDestActive())
+				if (directoryDestActive()) {
 					attach_filePaths(swdef, SW_A_source, source, path);
-				else 
+				} else {
 					attach_filePaths(swdef, SW_A_source, source, source);
-				//
-				// Don't stat the source
-				//
-				if (do_no_stat) *do_no_stat = 1;
+				}
+				SWEXTDEF_DEBUG("type d: case 1");
+				if (local_access(source, F_OK)) {
+					//
+					// not found or no access.
+					// Don't stat the source
+					//
+					if (do_no_stat) *do_no_stat = 1;
+					non_existing_sourceM = 1;
+					SWEXTDEF_DEBUG2("%s has no access", source);
+	
+					//
+					// apply implementation defined values
+					// per Section 5.2.13.3 Lines 831-836
+					//
+				} else {
+					//
+					// file exists
+					//
+					SWEXTDEF_DEBUG2("%s does have access",source);
+					non_existing_sourceM = 0;
+					if (do_no_stat) *do_no_stat = 0;
+				}
 			} else if (is_path_true(o_source) && is_path_true(o_path)) {
 				//
 				// 5.2.13.3: Lines 832-833.
 				// Do stat the source.
 				//
 				attach_filePaths(swdef, SW_A_source, source, path);
-				if (do_no_stat) *do_no_stat = 0;
+
+				if (local_access(source, F_OK)) {
+					SWEXTDEF_DEBUG2("%s has no access", source);
+					non_existing_sourceM = 1;
+					if (do_no_stat) *do_no_stat = 1;
+				} else {
+					SWEXTDEF_DEBUG2("%s does have access", source);
+					non_existing_sourceM = 0;
+					if (do_no_stat) *do_no_stat = 0;
+				}
+				SWEXTDEF_DEBUG("type d: case 2");
 			} else {
+				SWEXTDEF_DEBUG("type d: case 3");
 				fprintf(stderr, "swpackage: invalid explicit file definition for type 'd'\n");
 				return -1;
 			}
-			if (generate_ExplicitFileAttributes(source, source, st, 0)) return -1;
+
+			is_file_attribute_sufficient(do_no_stat, fileArrayWasSetM, fileDefPermsArrayWasSetM);
+			//
+			// *do_no_stat has now been modifed by the above function.
+			//
+
+			if (*do_no_stat) {
+				//
+				// no file to stat, therefore 
+				// apply implementation defined values
+				// per Section 5.2.13.3 Lines 831-836
+				//
+				;
+				if (apply_default_values(fileArrayM[typeE][0]))
+					return -1;
+			}
+			if (generate_ExplicitFileAttributes(source, source, st, !(*do_no_stat))) return -1;
 		} else if (fileArrayM[typeE][0] == SW_ITYPE_h) {	
 			//
 			// Hard Link: 5.2.13.3: Lines 837-840.
 			//
-
 			SWEXTDEF_DEBUG("In type h");
+			if (local_access(source, F_OK)) {
+				//
+				// not found or no access.
+				// Don't stat the source
+				//
+				if (do_no_stat) *do_no_stat = 1;
+				non_existing_sourceM = 1;
+				SWEXTDEF_DEBUG2("%s has no access", source);
+
+				//
+				// apply implementation defined values
+				// per Section 5.2.13.3 Lines 831-836
+				//
+			} else {
+				//
+				// file exists
+				//
+				SWEXTDEF_DEBUG2("%s does have access",source);
+				non_existing_sourceM = 0;
+				if (do_no_stat) *do_no_stat = 0;
+			}
+
 			if (!is_path_true(o_source) || !is_path_true(o_path)) {
 				fprintf(stderr, "swpackage: invalid explicit file definition for type 'h'\n");
 				return -1;
@@ -2508,10 +2751,30 @@ private: // ================================================================
 				strob_strcpy(file_linksource_pathM, o_path);
 			}
 
+			//
+			// There are three ways to handle this:
+			//  1) Add a line in the "as now theoretical-non-existing" _install script
+			//     and do not include in the storage section.
+			//         ln source path 
+			//         # The "ln' command uses the file attributes of the source
+			//	   # which is what we want to happen (presumably, absent other information).
+			//  2) Specify and require all the file attributes are specified
+			//     (because there is no safe reasonable way to get default values??
+			//  3) Guess at defaults, assume the hard link source is a regular file.
+			//
+			// For now we will use 3)
+
+			if (*do_no_stat) {
+				// No file to stat
+				// Assume regular file
+				if (apply_default_values((int)(SW_ITYPE_f)))
+					return -1;
+			}
+
 			attach_filePaths(swdef, "link_source", strob_str(file_linksource_pathM), (char*)(NULL));
 			attach_filePaths(swdef, SW_A_source, o_source, strob_str(file_pathM));
-			if (generate_ExplicitFileAttributes(source, (char *)NULL, st, 1)) return -1;
-			if (do_no_stat) *do_no_stat = 1;
+			if (generate_ExplicitFileAttributes(source, (char *)NULL, st, !(*do_no_stat))) return -1;
+
 		} else if (fileArrayM[typeE][0] == SW_ITYPE_s) {	
 			//
 			// Symbolic link: 5.2.13.3: Lines 841-844
@@ -2534,6 +2797,8 @@ private: // ================================================================
 			}
 			attach_filePaths(swdef, "link_source", strob_str(file_linksource_pathM), (char*)(NULL));
 			attach_filePaths(swdef, SW_A_source, o_source, strob_str(file_pathM));
+			if (apply_default_values(fileArrayM[typeE][0]))
+				return -1;
 			copy_file_cell(fileArrayM, fileArrayWasSetM, modeE, "0777");
 			if (generate_ExplicitFileAttributes(source, (char*)NULL, st, 0)) return -1;
 			if (do_no_stat) *do_no_stat = 1;
