@@ -231,27 +231,6 @@ class swPackageFile :  public swAttribute
 		return xFormat_write_file(source_fd);
 	}
 	
-	virtual int swfile_make_sums(swDefinition * swdef, char * sourcepath) {
-		int size;
-		unsigned long crc;
-		char digest[64];
-		char buf[64];
-		
-		if (swfile_get_posix_cksum(&crc)) return -1;
-		if (swfile_get_ascii_md5(digest)) return -2;
-		size = swfile_get_size();
-		if (size < 0) return -3;
-		
-		snprintf(buf, sizeof(buf), "%d", (int)(size));
-		buf[sizeof(buf) - 1] = '\0';
-		swdef->add("size", buf);
-		snprintf(buf, sizeof(buf), "%lu", crc);
-		buf[sizeof(buf) - 1] = '\0';
-		swdef->add("chksum", buf);
-		swdef->add("chksum_md5", digest);
-		return 0;
-	}
-	
 	virtual void swfile_set_default_statbuf(void) {
 		struct stat t;
 		struct stat * statbuf = &t;
@@ -762,7 +741,9 @@ fprintf(stderr, "JL no_stat = %d ========================================\n", no
 				// fstat's.
 				//
 				source_fd = xFormat_u_open_file(source);  // This sets the global stat/header buf.
-				if (source_fd < 0) return -4;
+				if (source_fd < 0)  {
+					return -4;
+				}
 			} else {
 				source_fd = -1;
 			}
