@@ -119,10 +119,12 @@ rpmpsf_write_swdef_internal(TOPSF * topsf, int uxfio_ofd, int swdef_filetype /*
 	Header h=topsf_get_rpmheader(topsf);
 	char * from_name=topsf_get_cwd_prefix(topsf);
 	char *name, buf[200];
-	char all_filesets[512];
+	STROB * all_filesets;
 	int count, type, filetype;
 	int ret = 0;
 
+	all_filesets = strob_open(100);
+
 	headerGetEntry(h, RPMTAG_SOURCERPM, &type, (void **) &name, &count);
 	if (!name && topsf->single_fileset_ == 0) {
 		filetype = RPMPSF_FILE_PSF_SRC;
@@ -167,6 +169,7 @@ rpmpsf_write_swdef_internal(TOPSF * topsf, int uxfio_ofd, int swdef_filetype /*
 	if (ret) {
 		SWBIS_ERROR_IMPL();
 	}
+	strob_close(all_filesets);
 	return ret;
 }
 
@@ -376,15 +379,15 @@ rpmpsf_get_rpmtagvalue(Header h, int tagnumber, int index, int *pcount, STROB *
 	}
 	if (hap[arrindex].header == h &&
 	    hap[arrindex].data != 0 &&
-	(swdef_maptable[arrindex].rpmtag_type == RPM_STRING_ARRAY_TYPE ||
-	 swdef_maptable[arrindex].rpmtag_type == RPM_I18NSTRING_TYPE
+	(swdef_pkg_maptable[arrindex].rpmtag_type == RPM_STRING_ARRAY_TYPE ||
+	 swdef_pkg_maptable[arrindex].rpmtag_type == RPM_I18NSTRING_TYPE
 	    )
 	    ) {
 
 		if (index >= hap[arrindex].count || index < 0) {
 			/*
 			fprintf(stderr,"get_rpmtagvalue(): rejecting %s lookup due to out of range index [%d].\n", 
-					swdef_maptable[arrindex].rpmtag_name, index);
+					swdef_pkg_maptable[arrindex].rpmtag_name, index);
 			*/
 			return -1;
 		}
@@ -399,8 +402,8 @@ rpmpsf_get_rpmtagvalue(Header h, int tagnumber, int index, int *pcount, STROB *
 
 	} else {
 
-		if (swdef_maptable[arrindex].rpmtag_type == RPM_STRING_ARRAY_TYPE ||
-		    swdef_maptable[arrindex].rpmtag_type == RPM_I18NSTRING_TYPE
+		if (swdef_pkg_maptable[arrindex].rpmtag_type == RPM_STRING_ARRAY_TYPE ||
+		    swdef_pkg_maptable[arrindex].rpmtag_type == RPM_I18NSTRING_TYPE
 		    ) {
 			if (hap[arrindex].data != 0) {
 				swbis_free(hap[arrindex].data);
@@ -418,12 +421,12 @@ rpmpsf_get_rpmtagvalue(Header h, int tagnumber, int index, int *pcount, STROB *
 		if (count <= index || index < 0) {
 			/*
 			fprintf(stderr,"get_rpmtagvalue():  %s lookup out of range index [%d].\n", 
-					swdef_maptable[arrindex].names[SWATMAP_NAME_RPMTAG], index);
+					swdef_pkg_maptable[arrindex].names[SWATMAP_NAME_RPMTAG], index);
 					SWATMAP_NAME_RPMTAG
 			*/
 			return -1;
 		}
-		if (type /*swdef_maptable[arrindex].rpmtag_type*/ == RPM_STRING_ARRAY_TYPE) {
+		if (type /*swdef_pkg_maptable[arrindex].rpmtag_type*/ == RPM_STRING_ARRAY_TYPE) {
 			hap[arrindex].header = h;
 			hap[arrindex].data = da;
 			hap[arrindex].count = count;
@@ -435,7 +438,7 @@ rpmpsf_get_rpmtagvalue(Header h, int tagnumber, int index, int *pcount, STROB *
 				ascii_copy(strb, entry);
 			}
 			return 0;
-		} else if (type /*swdef_maptable[arrindex].rpmtag_type*/ == RPM_I18NSTRING_TYPE) {
+		} else if (type /*swdef_pkg_maptable[arrindex].rpmtag_type*/ == RPM_I18NSTRING_TYPE) {
 			if (count == 1) {
 				ct = 0;
 				/*entry = (char *)da; */
@@ -458,7 +461,7 @@ rpmpsf_get_rpmtagvalue(Header h, int tagnumber, int index, int *pcount, STROB *
 				}
 				return 0;
 			}
-		} else if (type /* swdef_maptable[arrindex].rpmtag_type*/ == RPM_STRING_TYPE) {
+		} else if (type /* swdef_pkg_maptable[arrindex].rpmtag_type*/ == RPM_STRING_TYPE) {
 			hap[arrindex].header = 0;
 			hap[arrindex].data = 0;
 			hap[arrindex].count = 0;
@@ -659,6 +662,7 @@ rpmpsf_write_vendor(TOPSF * topsf, int uxfio_ofd, int filetype)
 		return 0;
 	}
 	write_object_keyword("vendor", uxfio_ofd);
+	ret += rpmpsf_write_rpm_attribute("the_term_vendor_is_misleading", "true", -1, 0, h, NULL, uxfio_ofd);
 	ret += rpmpsf_write_rpm_attribute("tag", name, -1, 0, h, NULL, uxfio_ofd);
 	{
 	switch(0) {
@@ -681,13 +685,11 @@ rpmpsf_write_vendor(TOPSF * topsf, int uxfio_ofd, int filetype)
 		return 0;
 	}
 	write_object_keyword("vendor", uxfio_ofd);
-	strncpy(tagstring, name, 63);
+	strncpy(tagstring, name, sizeof(tagstring));
 	tagstring[sizeof(tagstring) -1] = '\0';
 	p1 = tagstring;
-	while (*(p1++)) {
-		if (*(p1 - 1) == ' ')
-			*(p1 - 1) = '_';
-	}
+	swlib_tr(tagstring, (int)('_'), (int)(' '));
+	ret += rpmpsf_write_rpm_attribute("the_term_vendor_is_misleading", "true", -1, 0, h, NULL, uxfio_ofd);
 	ret += rpmpsf_write_rpm_attribute("tag", tagstring, -1, 0, h, NULL, uxfio_ofd);
 	ret += rpmpsf_write_rpm_attribute("title", (char *) NULL, RPMTAG_VENDOR, 0, h, NULL, uxfio_ofd);
 	ret += rpmpsf_write_rpm_attribute("qualifier", "seller", -1, 0, h, NULL, uxfio_ofd);
@@ -698,6 +700,7 @@ rpmpsf_write_vendor(TOPSF * topsf, int uxfio_ofd, int filetype)
 		return 0;
 	}
 	write_object_keyword("vendor", uxfio_ofd);
+	ret += rpmpsf_write_rpm_attribute("the_term_vendor_is_misleading", "true", -1, 0, h, NULL, uxfio_ofd);
 	ret += rpmpsf_write_rpm_attribute("tag", (char *) NULL, RPMTAG_PACKAGER, 0, h, NULL, uxfio_ofd);
 	ret += rpmpsf_write_rpm_attribute("title", (char *) NULL, RPMTAG_PACKAGER, 0, h, NULL, uxfio_ofd);
 	ret += rpmpsf_write_rpm_attribute("qualifier", "packager", 0, 0, h, NULL, uxfio_ofd);
@@ -706,14 +709,14 @@ rpmpsf_write_vendor(TOPSF * topsf, int uxfio_ofd, int filetype)
 }
 
 int
-rpmpsf_write_product(TOPSF * topsf, int uxfio_ofd, char *all_filesets, int filetype)
+rpmpsf_write_product(TOPSF * topsf, int uxfio_ofd, STROB * all_filesets, int filetype)
 {
 	Header h=topsf_get_rpmheader(topsf);
 	char * cdir;
-	char tagstring[256];
 	char *name, *p1;
 	int count, type;
 	int ret = 0;
+	STROB * tagstring = strob_open(48);
 	STROB * rpmtagname = strob_open(1);
 	STROB * tmp = strob_open(1);
 	STROB * rev = strob_open(1);
@@ -724,27 +727,21 @@ rpmpsf_write_product(TOPSF * topsf, int uxfio_ofd, char *all_filesets, int filet
 	if (!name)
 		return -1;
 	strob_strcpy(rpmtagname, name);
-	swlib_strncpy(tagstring, name, 256);
-	tagstring[255] = '\0';
+	strob_strcpy(tagstring, name);
 
-	if (filetype == RPMPSF_FILE_PSF_BIN)  cdir = ""; else cdir = tagstring; /* This line has no effect */
-	cdir = tagstring;
+	cdir = strob_str(tagstring);
 	ret += rpmpsf_write_rpm_attribute("control_directory", cdir, -1, 0, h, NULL, uxfio_ofd);
 
 	/* Use the vendor as a base for the vendor_tag */
 	headerGetEntry(h, RPMTAG_VENDOR, &type, (void **) &name, &count);
 	if (!name) {
-		strncpy(tagstring, "unknown", 63);
+		strob_strcpy(tagstring, "unknown");
 	} else {
-		strncpy(tagstring, name, 63);
-		tagstring[sizeof(tagstring) - 1] = '\0';
+		strob_strcpy(tagstring, name);
 	}
-	p1 = tagstring;
-	while (*(p1++)) {
-		if (*(p1 - 1) == ' ')
-			*(p1 - 1) = '_';
-	}
-	
+	p1 = strob_str(tagstring);
+	swlib_tr(strob_str(tagstring), (int)('_'), (int)(' '));
+
 	if (rpmpsf_get_rpmtagvalue(h, RPMTAG_VERSION, 0, (int *)NULL, tmp)) {
 		SWBIS_ERROR_IMPL();
 		fprintf(stderr, "rpmpsf.c error getting RPMTAG_VERSION\n");	
@@ -765,23 +762,32 @@ rpmpsf_write_product(TOPSF * topsf, int uxfio_ofd, char *all_filesets, int filet
 	ret += rpmpsf_write_rpm_attribute("mod_time", NULL, RPMTAG_INSTALLTIME, -1, h, NULL, uxfio_ofd);
 
 	if (filetype == RPMPSF_FILE_PSF_BIN) {
-		/* use a constant and uncommon  name for the fileset */
-		/* JL 2004-05-05 strncpy(tagstring, RPMPSF_BIN_FILESET_TAG, 63); */
-		/* strob_sprintf(tmp, 0, "%s-RUN", strob_str(rpmtagname)); */
-		strob_sprintf(tmp, 0, "%s", strob_str(rpmtagname));
-		strncpy(tagstring, strob_str(tmp), 63);
-		tagstring[63] = '\0';
-		ret += rpmpsf_write_rpm_attribute("all_filesets", tagstring, -1, 0, h, NULL, uxfio_ofd);
+		/*
+		 * Gaurd against illegal characters in the tag string here
+		 */
+		strob_strcpy(tagstring, strob_str(rpmtagname));
+		swlib_tr(strob_str(tagstring), (int)('_'), (int)('.'));  /* '.' are not allowed in tags */
+		swlib_tr(strob_str(tagstring), (int)('_'), (int)('/'));  /* '/' are not allowed in tags */
+		if (swlib_is_clean_path(strob_str(tagstring)) != 0) {
+			/*
+			 * Fatal
+			 */
+			return -1;
+		}
+		ret += rpmpsf_write_rpm_attribute("all_filesets", strob_str(tagstring), -1, 0, h, NULL, uxfio_ofd);
 	} else if (filetype == RPMPSF_FILE_PSF_SRC) {
-		/* the filesets will be: source, build_control, patches */
-		swlib_strncpy(tagstring, RPMPSF_FILESET_SOURCE, 100);
-		strcat(tagstring, " ");
-		strncat(tagstring, RPMPSF_FILESET_BUILD, 60);
-		strcat(tagstring, " ");
-		strncat(tagstring, RPMPSF_FILESET_PATCHES, 60);
-		ret += rpmpsf_write_rpm_attribute("all_filesets", tagstring, -1, 0, h, NULL, uxfio_ofd);
+		/*
+		 * the filesets will be: source, build_control, patches
+		 */
+		strob_strcpy(tagstring, RPMPSF_FILESET_SOURCE);
+		strob_strcat(tagstring, " ");
+		strob_strcat(tagstring, RPMPSF_FILESET_BUILD);
+		strob_strcat(tagstring, " ");
+		strob_strcat(tagstring, RPMPSF_FILESET_PATCHES);
+		ret += rpmpsf_write_rpm_attribute("all_filesets", strob_str(tagstring), -1, 0, h, NULL, uxfio_ofd);
 	}
-	swlib_strncpy(all_filesets, tagstring, 512);
+	strob_strcpy(all_filesets, strob_str(tagstring));
+	strob_close(tagstring);
 	strob_close(rpmtagname);
 	strob_close(tmp);
 	strob_close(rev);
@@ -789,13 +795,13 @@ rpmpsf_write_product(TOPSF * topsf, int uxfio_ofd, char *all_filesets, int filet
 }
 
 int
-rpmpsf_write_filesets(TOPSF * topsf, int uxfio_ofd, char *all_filesets, int filetype, int swdef_filetype)
+rpmpsf_write_filesets(TOPSF * topsf, int uxfio_ofd, STROB * all_filesets, int filetype, int swdef_filetype)
 {
 	int ret = 0, i = 0;
 	STROB * tmp = strob_open(1);
 	char *buf, *p;
 
-	while ((p = rpmpsf_list_iterate((i++ == 0) ? (&buf) : NULL, all_filesets, " ")) && p) {
+	while ((p = rpmpsf_list_iterate((i++ == 0) ? (&buf) : NULL, strob_str(all_filesets), " ")) && p) {
 		if (filetype == RPMPSF_FILE_PSF_BIN) {
 			strob_sprintf(tmp, 0, "%s-RUN", p);
 		} else {
@@ -976,6 +982,7 @@ rpmpsf_write_rpm_attribute(char *kw, char *value, int rpmtag, int index, Header
 {
 	static STROB *strb;
 	char *towrite = NULL;
+	STROB * tmp;
 	int count, i = 0;
 	if (!strb)
 		strb = strob_open(16);
@@ -1022,8 +1029,19 @@ rpmpsf_write_rpm_attribute(char *kw, char *value, int rpmtag, int index, Header
 	}
 
 	if (kw && towrite) {
-		write_keyword(kw, uxfio_ofd);
-		swdef_write_value(towrite, uxfio_ofd, 0, 0);
+		if (strcmp(kw, SW_A_tag) == 0) {
+			/*
+			 * translate '.' to '_'
+			 */
+			tmp = strob_open(32);
+			strob_strcpy(tmp, towrite);
+			swlib_tr(strob_str(tmp), (int)('_'), (int)('.'));
+			write_keyword(kw, uxfio_ofd);
+			swdef_write_value(strob_str(tmp), uxfio_ofd, 0, 0);
+		} else {
+			write_keyword(kw, uxfio_ofd);
+			swdef_write_value(towrite, uxfio_ofd, 0, 0);
+		}
 	}
 	return 0;
 }
