@@ -57,7 +57,6 @@ swdef_write_keyword_to_buffer(STROB * ob, char *keyword, int level,
 		int attribute_type)
 {
 	char sp[8];
-	int i;
 
 	memset(sp, 040, sizeof(sp));
 	if (level > (int)sizeof(sp)) level = 1;
@@ -93,7 +92,7 @@ OLDswdef_write_keyword(char *keyword, int level,
 			int attribute_type, int uxfio_fd)
 {
 	char sp[40];
-	int i, iret = 0, ret = 0;
+	int iret = 0, ret = 0;
 
 	memset(sp, 040, sizeof(sp));
 	if (level > (int)sizeof(sp)) level = sizeof(sp);
