@@ -35,8 +35,9 @@ swgp_stdioPump(int stdoutfd,
 		unsigned long int * statbytes);
 /* int swgp_pump_line(int ofd, int ifd); */
 int swgp_read_line(int fd, STROB * buf);
-void swgp_close_all_fd(int start);
+int swgp_close_all_fd(int start);
 int swgpReadFdNonblock(char * pipe_buf, int efd_from, int * len);
 int swgpReadLine(STROB * buf, int fd_from, int * readReturn);
+void swgp_check_fds(void);
 
 #endif
