@@ -356,11 +356,12 @@ swheader_set_current_offset_p(SWHEADER * swheader, int *n)
 		swheader->current_offset_p_=n;
 }
 
-o__inline__
-void *
+/* void * */
+/* (char * (*)(void *, int*)) */
+void * 
 swheader_get_iter_function(SWHEADER * swheader)
 {
-	return (swheader->f_goto_next_);
+	return (void*)(swheader->f_goto_next_);
 }
 
 /*
