@@ -105,7 +105,7 @@ swheaderline_get_type(char *output_line)
 char *
 swheaderline_get_keyword(char *output_line)
 {
-	char *t, *ret, *outputline;
+	char *t, *ret;
 
 	if (!output_line) return (char*)(NULL);
 
@@ -151,12 +151,10 @@ swheaderline_get_keyword(char *output_line)
 int
 swheaderline_get_value_length(char *output_line)
 {
-	int i, j, m;
+	int i;
 	char  * t, *p;
 	int n;
 	int mytens[6] = {0, 1, 10, 100, 1000, 10000};
-	int otens[6] = {0, 1, 10, 100, 1000, 10000};
-	int mydigs[7];
 
 	if (!isdigit((int)(*output_line))) {
 		fprintf(stderr, "bad parser output, value length not found\n"); 
