@@ -156,7 +156,6 @@ static
 int
 printlogline(STROB * buf, char * sptr)
 {
-	int ret;
 	time_t tm;
 	char result[64];
 	char * s;
@@ -365,9 +364,6 @@ int
 swlib_writelogline(int logfd, char * sptr)
 {
 	int ret;
-	time_t tm;
-	char result[64];
-	char * s;
 	STROB * buf = strob_open(64);
 	printlogline(buf, sptr);
 	ret = write(logfd, strob_str(buf), strob_strlen(buf));
@@ -523,6 +519,22 @@ swlib_is_ascii_noaccept(char * str, char * acc, int minlen) {
 	return 0;
 }
 
+int
+swlib_tr(char * src, int to, int from)
+{
+	int ret = 0;
+	char * p1;
+
+	p1 = src;
+	while (*(p1++)) {
+		if (*(p1 - 1) == from) {
+			*(p1 - 1) = to;
+			ret ++;
+		}
+	}
+	return ret;
+}
+
 int
 swlib_is_safe_path(char * s)
 {
@@ -1329,7 +1341,6 @@ swlib_doif_writef(int verbose_level, int write_at_level,
 			struct sw_logspec * logspec, int fd, char * format, ...)
 {
 	STROB * buffer;
-	int ret;
 	int newret;
 	va_list ap;
 	int level;
@@ -1354,14 +1365,6 @@ swlib_doif_writef(int verbose_level, int write_at_level,
 	}
 
 	if (logspec && logspec->logfdM > 0) {
-		/*
-		level = logspec->loglevelM;
-		if (level > 0) level += 2;
-		if (
-			write_at_level > 0 && 
-			level >= write_at_level
-		) {
-		*/
 		if (swlib_do_log_is_true(logspec, verbose_level, write_at_level)) {
 			/*
 			* The only lines logged here are lines that
@@ -1738,7 +1741,6 @@ swlib_md5_copy(int ifd, int count, char * md5, int ofd)
 	int n;
 	unsigned char buf[4096];
 	int blocksize = 4096;  /* this must match the sha_block size */
-	int digest_hex_bytes = 40;
 	int amount;
 	int readamount;
 	int ret;
@@ -1746,7 +1748,6 @@ swlib_md5_copy(int ifd, int count, char * md5, int ofd)
 	int am;
 	char * p5;
 	MD5_CTX md5ctx;
-	char * p;
 
 	swlib_md5_from_memblocks(&md5ctx, md5, (unsigned char*)NULL, -1);
 
@@ -1921,12 +1922,25 @@ swlib_drop_root_privilege(void)
 	int ret = 1;
 	if (getuid() == (uid_t)(0)) {
 		uid_t nob;
+		/*
+		 * Only attempt to drop root privilidge if
+		 * you are root
+		 */
 		if (taru_get_uid_by_name("nobody", &nob) < 0) {
 			fprintf(stderr, 
 	     "%s: Warning: the uname [nobody] not found, not dropping privilege.\n",
 			swlib_utilname_get());
 			ret = 3;
 		} else {
+			if (nob < 10) {
+				/*
+				 * This is a sanity check, which is only a warning
+				 * if it fails.  It assumes that the uid of nobody is >10
+				 * this to guard against a hacked /etc/group file.
+				 */
+				fprintf(stderr, "%s: User name 'nobody' has a uid of %d\n",
+				swlib_utilname_get(), (int)(nob));
+			}
 			if(setuid(nob) < 0) {
 				fprintf(stderr, 
 	     			"%s: setuid(uid=%d) failed at %s:%d: %s\n",
