@@ -31,6 +31,7 @@
 #include "xformat.h"
 #include "swutilname.h"
 #include "rpmpsf.h"
+#include "um_header.h"
 
 static int print_links(TOPSF * topsf);
 static void topsf_i_package_close(TOPSF * topsf);
@@ -39,7 +40,6 @@ static TOPSF * topsf_i_package_open(TOPSF * topsf, int oflags);
 static int topsf_rpm_find_linkname(TOPSF * topsf, char * baselink, int fd, struct new_cpio_header *file_hdr, int format);
 static int topsf_rpm_do_audit_hard_links(TOPSF * topsf, int fd, int oflags, char * decomp);
 
-
 static
 int
 find_link(TOPSF * topsf, char * keyname)
@@ -99,6 +99,7 @@ TOPSF * topsf_open(char * package_name, int oflags)
 	topsf->form_ = 0;
 	topsf->control_directoryM = strob_open(10);
 	topsf->taruM = taru_create();
+	time(&(topsf->mtimeM));
 	return topsf_i_package_open(topsf, oflags);
 }
 
@@ -128,6 +129,8 @@ topsf_i_package_open(TOPSF * topsf, int oflags) {
 	int rc, isSource;
 	STROB * tagval = strob_open(12);
 	int tagret = -1;
+	int deb_data_fd;
+	int ret;
 
 	SWBIS_E_DEBUG("FUNCTION ENTER");
 	switch (uinfile_get_type(topsf->format_desc_)) {
@@ -156,6 +159,29 @@ topsf_i_package_open(TOPSF * topsf, int oflags) {
 					return NULL;
 				}
 			}
+			break;
+		case DEB_FILEFORMAT:
+				/* fprintf(stderr, "lxpsf: deb format detected.\n"); */
+				topsf->rpmfd_ = NULL;
+				swlib_pipe_pump(STDOUT_FILENO, topsf->fd_);
+				ret = uxfio_lseek(topsf->format_desc_->underlying_fdM, TARU_AR_HEADER_SIZE+1, SEEK_CUR);
+				SWLIB_ASSERT(ret >= 0);
+				deb_data_fd = topsf->format_desc_->underlying_fdM;
+				swlib_pipe_pump(-1/*STDOUT_FILENO*/, deb_data_fd);
+			break;
+				/*
+				rc = topsf_deb_read_control_archive(topsf, topsf->fd_);
+				if (topsf_deb_do_audit_hard_links(topsf, topsf->fd_, oflags)) {
+					SWBIS_E_DEBUG("ABNORMAL FUNCTION EXIT");
+					strob_close(tagval);
+					return NULL;
+				}
+				*/
+				/*
+				 * the package file descriptor is now at the end of the
+				 * control.tar.gz file
+				 */
+
 			break;
 		default:
 				fprintf(stderr, "lxpsf: unrecognized package format.\n");
@@ -176,6 +202,9 @@ topsf_i_package_close(TOPSF * topsf) {
 	switch (uinfile_get_type(topsf->format_desc_)) {
 		case RPMRHS_FILEFORMAT:
 			headerFree(topsf->h_);
+		break;
+		case DEB_FILEFORMAT:
+		
 		break;
 		default:
 			fprintf(stderr, "unrecognized format in topsf_close().\n");
@@ -191,6 +220,12 @@ topsf_get_fd(TOPSF * topsf)
 	return topsf->fd_;
 }
 
+void 
+topsf_set_mtime(TOPSF * topsf, time_t tm)
+{
+	topsf->mtimeM = tm;
+}
+
 o__inline__
 void 
 topsf_set_fd(TOPSF * topsf, int fd)
@@ -232,7 +267,7 @@ topsf_add_fl_entry(TOPSF * topsf, char * to_name, char * from_name, int source_c
 		if (!strstr(from_name, topsf->cwd_prefix_)) {
 			SWBIS_E_DEBUG("exception");
 		} else {
-			int i = strlen(topsf->cwd_prefix_);
+			/* int i = strlen(topsf->cwd_prefix_); */
 			from_name+=strlen(topsf->cwd_prefix_);
 			/* if (*from_name == '/') from_name++; */
 			from_name--;   /* backup one char to get off the '/' */
@@ -291,16 +326,10 @@ topsf_rpm_do_audit_hard_links(TOPSF * topsf, int fd, int oflags, char * decomp)
 	int uxfio_fd, pid;
 	int opipe[2];	
 	SHCMD *cmd[100];	
-	UINFORMAT *uinformat;
+	UINFORMAT * uinformat;
 	
 	SWBIS_E_DEBUG("FUNCTION ENTER");
 
-	/*
-	buffertype = UXFIO_BUFTYPE_FILE;
-	if (getenv("XMEM"))
-	buffertype = UXFIO_BUFTYPE_DYNAMIC_MEM;
-	*/
-
 	buffertype = uinfile_decode_buftype(oflags, UXFIO_BUFTYPE_DYNAMIC_MEM);
 
 	cmd[0]=shcmd_open();
@@ -410,7 +439,8 @@ topsf_rpm_audit_hard_links(TOPSF * topsf, int fd)
 		
 		/* FIXME check for eoa */
 		if ((file_hdr->c_mode & CP_IFMT) == CP_IFREG) {
-			if (taru_write_archive_member_data(topsf->taruM, file_hdr, -1, fd, (int(*)(int))NULL, format, -1, NULL) < 0){
+			if (taru_write_archive_member_data(topsf->taruM, file_hdr,
+						-1, fd, (int(*)(int))NULL, format, -1, NULL) < 0){
 				fprintf(stderr,"data error %s\n", ahsStaticGetTarFilename(file_hdr));
 				return -2;
 			}
@@ -481,7 +511,8 @@ topsf_rpm_find_linkname(TOPSF * topsf, char * base_link, int fd, struct new_cpio
 		}
 
 		if ((file_hdr1->c_mode & CP_IFMT) == CP_IFREG) {
-			if (taru_write_archive_member_data(topsf->taruM, file_hdr1, -1, fd, (int(*)(int))NULL, format, -1, NULL) < 0){
+			if (taru_write_archive_member_data(topsf->taruM, file_hdr1,
+						-1, fd, (int(*)(int))NULL, format, -1, NULL) < 0){
 				fprintf(stderr,"data error %s\n", ahsStaticGetTarFilename(file_hdr1));
 				exit(1);
 			}
@@ -489,7 +520,9 @@ topsf_rpm_find_linkname(TOPSF * topsf, char * base_link, int fd, struct new_cpio
 		}		
 		if (file_hdr1->c_ino == base_file_hdr->c_ino){  /* FIXME need to check the device */
 			if (lastmatch) swbis_free(lastmatch);	
-			if (topsf->debug_link_) fprintf(stderr, "*Found linked files %s %s\n", base_link, ahsStaticGetTarFilename(file_hdr1));
+			if (topsf->debug_link_)
+				fprintf(stderr, "*Found linked files %s %s\n",
+					base_link, ahsStaticGetTarFilename(file_hdr1));
 			add_link_pair(topsf, base_link, ahsStaticGetTarFilename(file_hdr1));
 			lastmatch=swlib_strdup(ahsStaticGetTarFilename(file_hdr1));
 		}
@@ -513,7 +546,6 @@ topsf_copypass_swacfl_header_list(TOPSF * topsf, XFORMAT * xxformat, SWVARFS * s
 {
 	int done;
 	int en_index=0;
-	int u_fd;
 	int pheader[2];
 	char * p;
 	char * savep;
@@ -521,20 +553,7 @@ topsf_copypass_swacfl_header_list(TOPSF * topsf, XFORMAT * xxformat, SWVARFS * s
 	struct new_cpio_header *file_hdr=ahsStaticCreateFilehdr();
 	swacfl_entry * en;
 	SWACFL *swacfl=topsf->swacfl_;
-	struct stat st;
 	int (*do_write_file)(TOPSF * topsf, int o_fd);
-	CPLOB * archive_names = topsf->archive_names_;
-	int archive_name_index = 0;
-	char * archive_name;
-
-	/*
-	xxformat = xformat_open(-1, ofd, (int)(arf_newascii));
-	swvarfs=swvarfs_opendup(topsf->fd_, 
-		UINFILE_DETECT_FORCEUXFIOFD | UINFILE_DETECT_NATIVE | UINFILE_UXFIO_BUFTYPE_DYNAMIC_MEM, (mode_t)(NULL));
-	if (!swvarfs) {
-		return -1;
-	}
-	*/
 
 	done = 0;
 	while (!done && (en=(swacfl_entry*)cplob_val(swacfl->entry_array_, en_index++))) {
@@ -558,7 +577,7 @@ topsf_copypass_swacfl_header_list(TOPSF * topsf, XFORMAT * xxformat, SWVARFS * s
 				file_hdr->c_gid=getgid();
 				file_hdr->c_nlink=1;
 				ahsStaticSetTarLinkname(file_hdr, NULL);
-				file_hdr->c_mtime=time(NULL);
+				file_hdr->c_mtime=topsf->mtimeM;
 				file_hdr->c_filesize=0;
 				file_hdr->c_namesize=strlen(en->from_name_) +1;
 				file_hdr->c_chksum=0;
@@ -617,7 +636,6 @@ topsf_copypass_swacfl_header_list(TOPSF * topsf, XFORMAT * xxformat, SWVARFS * s
 int
 topsf_copypass_swacfl_archive_list(TOPSF * topsf, XFORMAT * xxformat, SWVARFS * swvarfs, int ofd)
 {
-	int en_index=0;
 	int u_fd;
 	int pheader[2];
 	char * p;
@@ -632,22 +650,6 @@ topsf_copypass_swacfl_archive_list(TOPSF * topsf, XFORMAT * xxformat, SWVARFS *
 	int archive_name_index = 0;
 	char * archive_name;
 
-	/*
-	SWBIS_E_DEBUG("FUNCTION ENTER");
-	SWBIS_E_DEBUG("xformat_open");
-	xxformat = xformat_open(-1, ofd, (int)(arf_newascii)); 
-	SWBIS_E_DEBUG("after xformat_open");
-	SWBIS_E_DEBUG("before swvarfs_opendup");
-	swvarfs=swvarfs_opendup(topsf->fd_, 
-		UINFILE_DETECT_FORCEUXFIOFD | UINFILE_DETECT_NATIVE | UINFILE_UXFIO_BUFTYPE_DYNAMIC_MEM, (mode_t)(NULL));
-	SWBIS_E_DEBUG("after swvarfs_opendup");
-	SWBIS_E_DEBUG2("topsf->fd_\n%s", uxfio_dump_string(topsf->fd_));
-	if (!swvarfs) {
-		SWBIS_E_FAIL("swvarfs_opendup failed");
-		return -1;
-	}
-	*/
-
 	while ((archive_name = cplob_val(archive_names, archive_name_index++))) {
 		en = swacfl_find_entry(swacfl, archive_name);
 		if (!en) {
@@ -673,7 +675,9 @@ topsf_copypass_swacfl_archive_list(TOPSF * topsf, XFORMAT * xxformat, SWVARFS *
 				SWBIS_E_DEBUG("SWACFL_SRCCODE_ARCHIVE_STREAM #1");
 				swvarfs_u_fstat(swvarfs, u_fd, &st);	
 				SWBIS_E_DEBUG("SWACFL_SRCCODE_ARCHIVE_STREAM #2");
-				taru_statbuf2filehdr(file_hdr, &st, (char*)NULL, p/*en->from_name_*/, swvarfs_u_get_linkname(swvarfs, u_fd));
+				taru_statbuf2filehdr(file_hdr, &st,
+						(char*)NULL, p/*en->from_name_*/,
+						swvarfs_u_get_linkname(swvarfs, u_fd));
 				SWBIS_E_DEBUG("SWACFL_SRCCODE_ARCHIVE_STREAM #3");
 				file_hdr->c_uid=getuid();
 				file_hdr->c_gid=getgid();
@@ -697,7 +701,7 @@ topsf_copypass_swacfl_archive_list(TOPSF * topsf, XFORMAT * xxformat, SWVARFS *
 				file_hdr->c_gid=getgid();
 				file_hdr->c_nlink=1;
 				ahsStaticSetTarLinkname(file_hdr, NULL);
-				file_hdr->c_mtime=time(NULL);
+				file_hdr->c_mtime=topsf->mtimeM;
 				file_hdr->c_filesize=0;
 				file_hdr->c_namesize=strlen(en->from_name_) +1;
 				file_hdr->c_chksum=0;
@@ -803,6 +807,10 @@ topsf_make_package_prefix(TOPSF * topsf, char * cwdir)
 			SWBIS_E_DEBUG("FUNCTION EXIT");
 			return ret;
 			break;
+		case DEB_FILEFORMAT:
+			ret = strdup("");
+			return ret;
+			break;
 		default:
 			fprintf(stderr, "unrecognized package type.\n");
 			break;
@@ -829,6 +837,8 @@ topsf_write_psf(TOPSF * topsf, int fd_out, int do_indent)
 				return ret;
 			}
 			break;
+		case DEB_FILEFORMAT:
+			break;
 		default:
 			fprintf(stderr, "unrecognized package type.\n");
 			break;
@@ -862,4 +872,3 @@ topsf_write_info(TOPSF * topsf, int fd_out, int do_indent)
 	SWBIS_E_DEBUG("FUNCTION EXIT");
 	return -1;
 }
-
