@@ -59,7 +59,6 @@ mychecksum(void * tar_hdr, int len)
   unsigned long sum = 0;
   char *p = (char *) tar_hdr;
   char *q = p + len;
-  int i;
   while (p < q)
     sum += *p++ & 0xff;
   return sum;
@@ -187,19 +186,11 @@ int
 decode_filter(XFORMAT * package, int verbose)
 {
 	int nullfd;
-	int format;
-	int output_format;
 	char * name;
 	int ifd;
 	int ret;
 	int aret;
-	int parseret;
-	int bytesret;
-	int is_dir;
-	int is_symlink;
-	int pathret;
 	int retval = -1;
-	int depth;
 	int ofd;
 	struct new_cpio_header * file_hdr;
 	STROB * namebuf;
@@ -374,8 +365,6 @@ main (int argc, char ** argv ) {
 	char * path;
 	char * t;
 	char * filename;
-	int opt_gnutar = 0;
-	int opt_star = 0;
 	int c;
 	int did_list = 0;
 	int filelistonly = 0;
