@@ -80,11 +80,8 @@ SWI_BASE *
 find_object_by_swsel(void * parent, char * swsel)
 {
 	int i;
-	int j;
 	SWI_PACKAGE * package;
 	SWI_BASE * object;
-	int ret;
-	int retval = 0;
 	char * verid_delim;
 
 	package = (SWI_PACKAGE *)(parent);
@@ -113,7 +110,6 @@ static
 int
 need_new_pfiles(SWI * swi)
 {
-	SWI_PACKAGE * package = swi->swi_pkgM;
 	SWPATH * swpath = swi->swpathM;
 	char * pfiles_dir;
 	char * pfiles_attribute;
@@ -182,7 +178,6 @@ swi_xfile_add_control_script(SWI * swi, SWI_XFILE * xfile, char * name,
 {
 	SWI_CONTROL_SCRIPT * sc;
 	SWI_FILE_MEMBER * s;
-	int offset;
 
 	sc = swi_control_script_create();
 	/* offset = swi->xformatM->swvarfsM->current_header_offsetM;*/
@@ -199,7 +194,6 @@ void
 swi_xfile_add_file_member(SWI * swi, SWI_XFILE * xfile, char * name, int fd)
 {
 	SWI_FILE_MEMBER * s;
-	int offset;
 	/* offset = swi->xformatM->swvarfsM->current_header_offsetM; */
 	s = swi_xfile_construct_file_member(xfile, name, fd,
 					swi->xformatM->swvarfsM);
@@ -428,10 +422,8 @@ swi_expand_shared_file_control_sripts(SWI * swi, SWI_XFILE * xfile)
 {
 	SWHEADER * swheader = xfile->headerM;
 	SWI_CONTROL_SCRIPT * script;
-	SWI_CONTROL_SCRIPT * source_script;
 	SWI_CONTROL_SCRIPT * new_script;
 	SWI_FILE_MEMBER * afile;
-	char * next_attr;
 	char * next_line;
 	char * path_attr;
 	char * path_value;
