@@ -66,7 +66,6 @@ char *
 swi_xfile_member_dump_string_s(SWI_FILE_MEMBER * xx, char * prefix)
 {
 	STROB * buf;
-	STROB * tmpbuf = strob_open(10);
 	char * datap;	
 	char * newline;	
 	if (buf1 == (STROB*)NULL) buf1 = strob_open(100);
@@ -183,7 +182,6 @@ swi_product_dump_string_s(SWI_PRODUCT * xx, char * prefix)
 	* SWI_XFILE * pfilesM;
 	* SWI_XFILE * swi_coM[SWI_MAX_OBJ]; 
 	*/
-	STROB * tmpprefix = strob_open(100);
 	int i = 0;
 	STROB * buf;
 	SWI_XFILE * xfile;
@@ -240,7 +238,6 @@ swi_product_dump_string_s(SWI_PRODUCT * xx, char * prefix)
 char * 
 swi_package_dump_string_s(SWI_PACKAGE * xx, char * prefix)
 {
-	SWPATH_EX * swexpath;
 	STROB * buf;
 	SWI_PRODUCT * prod;
 	int i;
@@ -297,10 +294,7 @@ swi_package_dump_string_s(SWI_PACKAGE * xx, char * prefix)
 char * 
 swi_dump_string_s(SWI * xx, char * prefix)
 {
-	SWPATH_EX * swexpath;
 	STROB * buf;
-	SWI_PRODUCT * prod;
-	int i;
 	char * tmp_s;
 	if (buf9 == (STROB*)NULL) buf9 = strob_open(100);
 	buf = buf9;
