@@ -285,6 +285,8 @@ get_pwent(
 			/*
 			 * The lookup was found in cache but it
 			 * may not be in /etc/passwd or /etc/group
+			 * i.e. The user is looking up a username that
+			 * is not in the system data files.
 			 */
 			*pid = n;
 			return is_in_sysdb;
@@ -309,6 +311,10 @@ get_pwent(
 		char * cname;
 		cname = return_name_by_cache(dbpair_len, cache, id, &is_in_sysdb);
 		if (cname) {
+			if (strlen(cname) >  TARU_SYSDBNAME_LEN - 1) {
+				fprintf(stderr, "%s: user name too long for ustar headers: %s\n", swlib_utilname_get(), cname);
+
+			}
 			strncpy(buf, cname, TARU_SYSDBNAME_LEN - 1);
 			buf[TARU_SYSDBNAME_LEN - 1] = '\0';
 			return is_in_sysdb;
@@ -565,7 +571,6 @@ taru_read_in_tar_header2(TARU * taru, struct new_cpio_header *file_hdr,
 	int warned = FALSE;
 	union tar_record tar_rec;
 	struct tar_header *tar_hdr = (struct tar_header *) &tar_rec;
-	unsigned char tmpchar;
 	char * aregfilename;
 	char * tmpname;
 	char * header_buffer = fheader_buffer;
