@@ -1075,9 +1075,6 @@ taruib_arfcopy(void * xpackage,
 	int ret;
 	int aret;
 	int parseret;
-	int bytesret;
-	int is_dir;
-	int is_symlink;
 	int pathret;
 	int retval = -1;
 	int depth;
