@@ -57,7 +57,6 @@
 #define UINFILE_I_INIT_READ_SIZE 124
 #define UINFILE_I_TAR_BLOCK_SIZE 512
 
-static sigset_t g_fork_blockmask;
 
 static 
 int
@@ -132,7 +131,6 @@ uinfile_i_get_name(UINFORMAT * uinformat,
 static
 int
 uinfile_check_ieee_fd(int fd) {
-	int ret = 0;
 	int current_buftype; 
 	current_buftype = uxfio_fcntl(fd, UXFIO_F_GET_BUFTYPE, 0);
 	if (
@@ -141,7 +139,7 @@ uinfile_check_ieee_fd(int fd) {
 		current_buftype != UXFIO_BUFTYPE_FILE
 		) {
 		fprintf(stderr,
-		"internal error, incorrect usage of uinfile_detect_ieee\n");
+		"internal error, incorrect usage of uinfile_check_ieee\n");
 		return -1;
 	}
 	if (current_buftype == UXFIO_BUFTYPE_MEM) {
@@ -157,7 +155,6 @@ uinfile_detect_ieee(UINFORMAT * uinformat, int oflags)
 {
 	int ret;
 	int nameretval;
-	int retval = 0;
 	char * name = NULL;
 	int mm=0;
 	int lead_bytes = 0;
@@ -172,9 +169,9 @@ uinfile_detect_ieee(UINFORMAT * uinformat, int oflags)
 	}
 
 	/* 
-	* Read the package and try to find
-	* the magic "catalog/INDEX" name 
-	*/
+	 * Read the package and try to find
+	 * the magic "catalog/INDEX" name 
+	 */
 
 	UINFILE_E_DEBUG("HERE");
 	/* 
@@ -220,7 +217,9 @@ uinfile_detect_ieee(UINFORMAT * uinformat, int oflags)
 			break;
 		}	
 		if ((file_hdr->c_mode & CP_IFMT) != CP_IFDIR) {
+			UINFILE_E_DEBUG("HERE");
 			if (allow_generic_tar == 0) {
+				UINFILE_E_DEBUG("HERE");
 				uinformat->layout_typeM = 
 						UINFILE_FILELAYOUT_UNKNOWN;
 				fprintf (stderr,
@@ -239,6 +238,7 @@ uinfile_detect_ieee(UINFORMAT * uinformat, int oflags)
 	}
 	if ((mm >= UINFILE_IEEE_MAX_LEADING_DIR &&
 			(allow_generic_tar == 0)) || name == NULL) {
+		UINFILE_E_DEBUG("HERE");
 		uinformat->layout_typeM = UINFILE_FILELAYOUT_UNKNOWN;
 		fprintf (stderr,
 		"%s: Package layout_version 1.0 not found.\n", swlib_utilname_get());
@@ -265,11 +265,6 @@ static
 int
 uinfile_handle_return(UINFORMAT * uinformat, int oflags, int uxfio_buffer_type)
 {
-	int ret;
-	int retval;
-	char * name;
-	int mm=0;
-	int lead_bytes = 0;
 	struct new_cpio_header *file_hdr=uinformat->file_hdrM;
 
 	UINFILE_E_DEBUG("");
@@ -279,7 +274,9 @@ uinfile_handle_return(UINFORMAT * uinformat, int oflags, int uxfio_buffer_type)
 	ahsStaticSetTarFilename(file_hdr, NULL);
 	ahsStaticSetTarLinkname(file_hdr, NULL);
 
-	if (uxfio_lseek(uinformat->fdM, 0L, SEEK_SET) < 0) {
+	UINFILE_E_DEBUG2("HERE current_pos_=%d", (int)((uinformat)->current_pos_));
+	if (uxfio_lseek(uinformat->fdM, (off_t)(0), SEEK_SET) < 0) {
+		UINFILE_E_DEBUG("HERE");
 		fprintf(stderr,
 		"uxfio_lseek error in uinfile_handle_return 0001. fd=%d\n",
 			uinformat->fdM);
@@ -287,8 +284,8 @@ uinfile_handle_return(UINFORMAT * uinformat, int oflags, int uxfio_buffer_type)
 	}
 	taruib_set_datalen(0);
 	/*
-	* Generic TAR or cpio archive.
-	*/
+	 * Generic TAR or cpio archive.
+	 */
 
 	UINFILE_E_DEBUG("HERE");
 	if (oflags & UINFILE_DETECT_FORCEUNIXFD && 
@@ -351,15 +348,16 @@ uinfile_i_open(char *filename, int oflag, mode_t mode, UINFORMAT ** uinformat,
 	int fd, pipe_fd[2], dupfd;
 	int refd;
 	int zpipe[2];
-	pid_t pid, zpid;
-	int bytesRead;
+	pid_t pid;
 	char buffer[512];
-	char gzmagic[] = "\x1f\x8b\x08\x00\x00";
-	char Zmagic[] = "\x1f\x9d\x90\x00\x00";
-	char rpmmagic[5] = "\xed\xab\xee\xdb\x00";
-	char bz2magic[4] = "\x42\x5a\x68\x00";
+	char gzmagic[] = UINFILE_MAGIC_gz;
+	char Zmagic[] = UINFILE_MAGIC_Z;
+	char rpmmagic[] = UINFILE_MAGIC_rpm;
+	char bz2magic[] = UINFILE_MAGIC_bz2;
+	char debmagic[] = UINFILE_MAGIC_deb;
 	char *zcat_command = (char *) (NULL);
 	pid_t a_pid;
+	int pump_amount = -1;
 	int zret = 0;	
 	int forcetar;
 	int doieee;
@@ -367,6 +365,10 @@ uinfile_i_open(char *filename, int oflag, mode_t mode, UINFORMAT ** uinformat,
 	int dounrpminstall;
 	int douncpio;
 	int peeklen;
+	int control_gz_size = 0;
+	int val1;
+	int val2;
+	char * s_size;
 
 	UINFILE_E_DEBUG("");
 	*uinformat=(UINFORMAT *)malloc(sizeof(UINFORMAT));
@@ -374,11 +376,14 @@ uinfile_i_open(char *filename, int oflag, mode_t mode, UINFORMAT ** uinformat,
 		return -1;
 	}
 	(*uinformat)->underlying_fdM = -1;
+	(*uinformat)->current_pos_ = 0;
 	(*uinformat)->zcat_cmdM = zcat_command;
 	(*uinformat)->file_hdrM=ahsStaticCreateFilehdr();
 	(*uinformat)->verboseM = 0;
 	(*uinformat)->has_leading_slashM = 0;
 	(*uinformat)->ztypeM = UINFILE_COMPRESSED_NA;
+	(*uinformat)->typeM = UNKNOWN_FILEFORMAT;
+	memset((*uinformat)->type_revisionM, '\0', sizeof((*uinformat)->type_revisionM));
 	(*uinformat)->swpathM = (SWPATH*)(NULL);
 	(*uinformat)->taruM = taru_create();
 	sigemptyset(&((*uinformat)->blockmask_));
@@ -405,25 +410,48 @@ uinfile_i_open(char *filename, int oflag, mode_t mode, UINFORMAT ** uinformat,
 		UINFILE_E_DEBUG("HERE PIPE");
 
 		if (filename == (char *) (NULL)) {
+			UINFILE_E_DEBUG("HERE");
 			dupfd = xdupfd;
 		} else {
+			UINFILE_E_DEBUG("HERE");
 			dupfd = STDIN_FILENO;
 		}
 		UINFILE_E_DEBUG2("HERE dupfd = %d", dupfd);
 		
-		if (dupfd < UXFIO_FD_MIN && lseek(dupfd, 0L, SEEK_CUR) == -1){
+		val1 = dupfd < UXFIO_FD_MIN ;
+		val2 = lseek(dupfd, 0L, SEEK_CUR);
+
+		if (val1 && val2 == -1) {
 			UINFILE_E_DEBUG3("HERE %d %s", errno, strerror(errno));
 			if (errno == ESPIPE){
-			UINFILE_E_DEBUG2("HERE\n%s", uxfio_dump_string(fd));
+				/*
+				 * unseekable pipe
+				 */
+				UINFILE_E_DEBUG2("HERE\n%s", uxfio_dump_string(fd));
 				fd = uxfio_opendup(dupfd, uxfio_buffer_type);
-			UINFILE_E_DEBUG2("HERE\n%s", uxfio_dump_string(fd));
-				if (fd < 0)
+				UINFILE_E_DEBUG2("HERE\n%s", uxfio_dump_string(fd));
+				if (fd < 0) {
+					fprintf(stderr,"uinfile.c: uxfio_opendup failed\n");
 					return -1;
+				}
 			} else {
 				perror("uinfile: fatal error");
 				return -1;
 			}
 		} else {
+			UINFILE_E_DEBUG("HERE");
+			ret = lseek(dupfd, 0L, SEEK_CUR);
+			if (ret >= 0) {
+				UINFILE_E_DEBUG2("HERE current_pos_=%d", (int)((*uinformat)->current_pos_));
+				(*uinformat)->current_pos_ = ret;
+				if ((*uinformat)->current_pos_) {
+					fprintf(stderr,"uinfile.c: warning: current_pos_ not equal to zero\n");
+				}
+			} else {
+				/*
+				fprintf(stderr,"uinfile.c: warning: lseek failed\n");
+				*/
+			}
 			fd = dupfd;
 		}
 		(*uinformat)->did_dupeM=1;
@@ -442,14 +470,22 @@ LABEL_REREAD:
 	(*uinformat)->fdM = refd;
 
 	/*
-	* Initial read on the serial access archive.
-	* Fixme:
-	*/
+	 * Initial read on the serial access archive.
+	 * Fixme:
+	 */
 	if (doarb == 0) {
+		UINFILE_E_DEBUG("HERE");
 		peeklen = UINFILE_I_INIT_READ_SIZE;
 	} else {
+		/*
+		 * this is used is DETECT_ABRITRARY_DATA is set
+		 */
+		UINFILE_E_DEBUG("HERE");
 		peeklen = 3;
 	}	
+
+	memset(buffer, '\0', sizeof(buffer));
+
 	if ((ret=taru_tape_buffered_read(refd, buffer, peeklen)) != peeklen) {
 		if (ret) fprintf(stderr,
 		"%s: error in initial read. request=%d return=%d\n",
@@ -459,15 +495,17 @@ LABEL_REREAD:
 	UINFILE_E_DEBUG("HERE");
 
 	/*
-	* Identify the file type.
-	*/
+	 * Identify the file type.
+	 */
 	if (doarb == 0) {
-		if (!memcmp(buffer, rpmmagic, 4) && !forcetar) {
+		if (0) {
+			;
+		} else if (!memcmp(buffer, rpmmagic, 4) && !forcetar) {
 			if (dounrpminstall == 0 && doieee == 0) {
 				/*
-				* This is the old historic response to
-				* an RPM file.
-				*/
+				 * This is the old historic response to
+				 * an RPM file.
+				 */
 				(*uinformat)->typeM=RPMRHS_FILEFORMAT;
 				ret = uinfile_handle_return(*uinformat, oflags,
 							uxfio_buffer_type);
@@ -475,10 +513,10 @@ LABEL_REREAD:
 				return ret;
 			} else if (dounrpminstall) {
 				/*
-				* Model the RPM as a compression type since this
-				* fits with the code below, e.g. instead of calling
-				* ''gzip -d'', call ''swpackage --unrpm''
-				*/
+				 * Model the RPM as a compression type since this
+				 * fits with the code below, e.g. instead of calling
+				 * ''gzip -d'', call ''swpackage --unrpm''
+				 */
 				(*uinformat)->ztypeM = UINFILE_COMPRESSED_RPM;
 			} else if (doieee && dounrpminstall == 0) {
 				fprintf(stderr,
@@ -486,15 +524,16 @@ LABEL_REREAD:
 						swlib_utilname_get());
 				return -1;
 			} else {
+				SWLIB_EXCEPTION("");
 				return -1;
 			}
 		} else if (douncpio && ((strncmp(buffer, "0707", 4) == 0) &&
 				(1/*strncmp(buffer + 257, TMAGIC, 5)*/))) {
 			/*
-			* Model the cpio format as a compression type since this
-			* fits with the code below, e.g. instead of calling
-			* ''gzip -d'', call ''arf2arf -H ustar''
-			*/
+			 * Model the cpio format as a compression type since this
+			 * fits with the code below, e.g. instead of calling
+			 * ''gzip -d'', call ''arf2arf -H ustar''
+			 */
 			(*uinformat)->ztypeM = UINFILE_COMPRESSED_CPIO;
 		} else if (!strncmp(buffer, "0707", 4) &&
 				(1/*strncmp(buffer + 257, TMAGIC, 5)*/) &&
@@ -507,6 +546,7 @@ LABEL_REREAD:
 			} else if (!strncmp(buffer, "070707", 6)) {/* POSIX.1 */
 				(*uinformat)->typeM = CPIO_POSIX_FILEFORMAT;
 			} else {
+				SWLIB_EXCEPTION("");
 				return -1;
 			}
 			(*uinformat)->has_leading_slashM =
@@ -528,37 +568,64 @@ LABEL_REREAD:
 		} else if (!memcmp(buffer, (void *) Zmagic, 2)) {
 							/* Unix compress */
 			(*uinformat)->ztypeM = UINFILE_COMPRESSED_Z;
+		} else if (!memcmp(buffer, (void *)debmagic, strlen(UINFILE_MAGIC_deb))) {
+							/* deb package */
+			(*uinformat)->ztypeM = UINFILE_COMPRESSED_DEB;
+			strncpy((*uinformat)->type_revisionM,
+				((char*)buffer)+68, 4);
+			/*
+			 * Now read some more bytes to include the gzip magic of
+			 * the control.tar.gz file
+			 */
+			if ((ret=taru_tape_buffered_read(refd, buffer+peeklen,
+					UINFILE_DEB_CONTROL_OFFSET - peeklen + 4)) != 
+				UINFILE_DEB_CONTROL_OFFSET - peeklen + 4) {
+				fprintf(stderr,
+				"%s: error in second initial read. request=%d return=%d\n",
+					swlib_utilname_get(), peeklen,  ret);
+				return -1;
+			}
 		} else {
 			/*
-			* If nothing special is detected, assume its tar.
-			*/
-			if ((*uinformat)->ztypeM == UINFILE_COMPRESSED_NA)
+			 * If nothing special is detected, assume its tar.
+			 */
+			if ((*uinformat)->ztypeM == UINFILE_COMPRESSED_NA) {
 				(*uinformat)->ztypeM = UINFILE_COMPRESSED_NOT;
+			}
 			if (taru_tape_buffered_read(refd,
-						buffer + peeklen,
-							512 - peeklen) 
-					!= (512 - peeklen)) 
-							return -1;
-			if (!strncmp(buffer + 257, TMAGIC, 5)) {
-							/* uncompressed tar */
-				(*uinformat)->typeM=USTAR_FILEFORMAT;
-				(*uinformat)->has_leading_slashM =
-					determine_if_has_leading_slash(
-								*uinformat,
-								buffer);
-				ret = uinfile_handle_return(*uinformat,
-						oflags, uxfio_buffer_type);
-				if (ret > 0 && doieee) {
-					ret = uinfile_detect_ieee(*uinformat,
-									oflags);
+				buffer + peeklen, 512 - peeklen) != (512 - peeklen)) return -1;
+			if (!strncmp(buffer + 257, "ustar" /*TMAGIC*/, 5)) {
+				/* uncompressed tar */
+				/*
+				 * This could be a deb format package
+				 */
+				if (
+					(*uinformat)->ztypeM == UINFILE_COMPRESSED_DEB
+				) {
+					(*uinformat)->typeM=DEB_FILEFORMAT;
+					(*uinformat)->has_leading_slashM = 0; /* NO */
+					ret = uinfile_handle_return(*uinformat,
+							oflags, uxfio_buffer_type);
+				} else {
+					(*uinformat)->typeM=USTAR_FILEFORMAT;
+					(*uinformat)->has_leading_slashM =
+						determine_if_has_leading_slash(
+									*uinformat,
+									buffer);
+					ret = uinfile_handle_return(*uinformat,
+							oflags, uxfio_buffer_type);
+					if (ret > 0 && doieee) {
+						ret = uinfile_detect_ieee(*uinformat,
+										oflags);
+					}
 				}
-			UINFILE_E_DEBUG2("HERE\n%s", uxfio_dump_string(ret));
+				UINFILE_E_DEBUG2("HERE\n%s", uxfio_dump_string(ret));
 				return ret;
 			} else {
 				uxfio_close(refd);
 				swbis_free(*uinformat);
 				fprintf(stderr,
-				"%s: error: unrecognized format\n",
+				"%s: error: uinfile: unrecognized format\n",
 						swlib_utilname_get());
 				return -1;
 			}
@@ -593,15 +660,70 @@ LABEL_REREAD:
 	unzip[0] = (SHCMD*)NULL;
 	unzip[1] = (SHCMD*)NULL;
 
-	if (uxfio_lseek(refd, 0L, SEEK_SET)) {
-		fprintf(stderr, "%s: %s: uxfio_lseek error. 001.1\n",
-			swlib_utilname_get(), __FILE__); 
+	UINFILE_E_DEBUG2("HERE current_pos_=%d", (int)((*uinformat)->current_pos_));
+	if (uxfio_lseek(refd, (off_t)((*uinformat)->current_pos_), SEEK_SET) !=
+		(off_t)((*uinformat)->current_pos_)
+	) {
+		fprintf(stderr, "%s: %s: uxfio_lseek error. 001.1 pos=%d\n",
+			swlib_utilname_get(), __FILE__, ((*uinformat)->current_pos_)); 
 		return -1;
 	}
 	if (zcat_command && (*uinformat)->ztypeM == UINFILE_COMPRESSED_Z) {
 		unzip[0] = shcmd_open();
 		shcmd_add_arg(unzip[0], "compress");
 		shcmd_add_arg(unzip[0], "-cd");
+	} else if ((*uinformat)->ztypeM == UINFILE_COMPRESSED_DEB) {
+		/*
+		 * Make some tests on the first few bytes of the package
+		 */
+		UINFILE_E_DEBUG("in deb code");
+		if (memcmp(((char*)(buffer)) + UINFILE_DEB_CONTROL_OFFSET,
+				gzmagic, 3) == 0) {
+			unzip[0] = shcmd_open();
+			shcmd_add_arg(unzip[0], "gzip");
+			shcmd_add_arg(unzip[0], "-d");
+		} else if (memcmp(((char*)(buffer)) + UINFILE_DEB_CONTROL_OFFSET,
+				bz2magic, 3) == 0) {
+			/*
+			 * preemptive support for bzip2'ed debian packages
+			 */
+			unzip[0] = shcmd_open();
+			shcmd_add_arg(unzip[0], "bzip2");
+			shcmd_add_arg(unzip[0], "-d");
+		} else {
+			/*
+			 * error, the compressed magic not found at the expected offset
+			 */
+			return -1;
+		} 
+		/*
+		 * grab the size of the control.tar.gz archive from member
+		 */
+		s_size = (char*)buffer + 
+				( strlen(UINFILE_MAGIC_ar) +
+				TARU_AR_HEADER_SIZE +
+				4 + /* member payload length i.e. "2.0\n" */
+				TARU_AR_SIZE_OFFSET );
+
+		control_gz_size = atoi(s_size);
+
+		if (control_gz_size <= 0 || control_gz_size > 100000000) {
+			/*
+			 * sanity fails
+			 */
+			return -1;
+		}
+
+		if (uxfio_lseek(refd, UINFILE_DEB_CONTROL_OFFSET, SEEK_SET) != UINFILE_DEB_CONTROL_OFFSET) {
+			fprintf(stderr, "%s: %s: uxfio_lseek error. 001.3 pos=%d\n",
+				swlib_utilname_get(), __FILE__, ((*uinformat)->current_pos_)); 
+			return -1;
+		} else {
+			/*
+			 * The file is now positioned at the control.tar.gz file
+			 */
+			; /* continue */
+		}
 	} else if ((*uinformat)->ztypeM == UINFILE_COMPRESSED_GZ) {
 		UINFILE_E_DEBUG("in gzip code");
 		unzip[0] = shcmd_open();
@@ -624,6 +746,7 @@ LABEL_REREAD:
 		/* shcmd_add_arg(unzip[0], "swbis"); */
 		shcmd_add_arg(unzip[0], SWBISBINDIR "/swpackage");
 		shcmd_add_arg(unzip[0], "--unrpm");
+		/*
 		shcmd_add_arg(unzip[0], "--no-defaults");
 		shcmd_add_arg(unzip[0], "-W");
 		shcmd_add_arg(unzip[0], "files");
@@ -633,6 +756,7 @@ LABEL_REREAD:
 		shcmd_add_arg(unzip[0], "archive-digests");
 		shcmd_add_arg(unzip[0], "-W");
 		shcmd_add_arg(unzip[0], "dummy-sign");
+		*/
 		shcmd_add_arg(unzip[0], "--catalog-owner=0");
 		shcmd_add_arg(unzip[0], "--catalog-group=0");
 		shcmd_add_arg(unzip[0], "@-");
@@ -668,21 +792,21 @@ LABEL_REREAD:
 		UINFILE_E_DEBUG2("HERE\n%s", uxfio_dump_string(refd));
 
 		/*
-		* Re-read the alledgedly uncompressed file.
-		*/
+		 * Re-read the alledgedly uncompressed file.
+		 */
 		goto LABEL_REREAD;
 	} else {
 		/* 
-		* Child 
-		*/
+		 * Child 
+		 */
 		int jlxx;
 		int b_closefd = -1;
 		char ** argvector;
 	
 		close(pipe_fd[0]);
 		/*
-		* This is the master child. clear all the pids.
-		*/
+		 * This is the master child. clear all the pids.
+		 */
 		for (i=0;
 			i < (int)(sizeof((*uinformat)->pidlistM)/sizeof(int));
 					i++)
@@ -698,10 +822,29 @@ LABEL_REREAD:
 			zpipe[1]=-1;
 
 			/*
-			* If its a uxfio descriptor then we must fork.
-			*/
-			if (refd >= UXFIO_FD_MIN) {
+			 * If its a uxfio descriptor then we must fork.
+			 */
+			if (
+				1 && (
+				refd >= UXFIO_FD_MIN ||
+				(*uinformat)->ztypeM == UINFILE_COMPRESSED_DEB
+				)
+			) {
 				UINFILE_E_DEBUG("HERE ");
+				if ((*uinformat)->ztypeM == UINFILE_COMPRESSED_DEB) {
+					/*
+					 * pump the exact size of the control.tar.gz file
+					 */
+					UINFILE_E_DEBUG("HERE");
+					pump_amount = control_gz_size;
+				} else {
+					/*
+					 * pump until the end
+					 */
+					UINFILE_E_DEBUG("HERE");
+					pump_amount = -1;
+				}
+
 				pipe(zpipe);
 				a_pid = swndfork(&((*uinformat)->blockmask_),
 						&((*uinformat)->defaultmask_));
@@ -714,13 +857,16 @@ LABEL_REREAD:
 				} else if (a_pid == 0) {
 					UINFILE_E_DEBUG("HERE ");
 					close(zpipe[0]);
-					uxfio_fcntl(refd,
+					if ((*uinformat)->ztypeM != UINFILE_COMPRESSED_DEB) {
+						uxfio_fcntl(refd,
 						UXFIO_F_ARM_AUTO_DISABLE, 1);
-					if (uxfio_lseek(refd, 0L, SEEK_SET) < 0)
-						fprintf(stderr,
-					"uxfio_lseek error in child 002.\n");
+						UINFILE_E_DEBUG2("HERE current_pos_=%d", (int)((*uinformat)->current_pos_));
+						if (uxfio_lseek(refd, (off_t)((*uinformat)->current_pos_), SEEK_SET) < 0)
+							fprintf(stderr,
+						"uxfio_lseek error in child 002.\n");
+					}
 					taru_pump_amount2(zpipe[1],
-								refd, -1, -1); 
+								refd, pump_amount, -1); 
 					close(zpipe[1]);
 					uxfio_close(refd);
 					_exit(0);
@@ -777,8 +923,8 @@ LABEL_REREAD:
 		}
 
 		/*
-		* Now wait on all the pids.
-		*/
+		 * Now wait on all the pids.
+		 */
 		if (b_closefd >= 0) {
 			uxfio_close(b_closefd);
 		}
@@ -957,14 +1103,6 @@ uinfile_set_type(UINFORMAT * uinformat, int type)
 	uinformat->typeM=type;
 }
 
-/* Unused function
-void
-uinfile_set_layout_type(UINFORMAT * uinformat, int type)
-{
-	uinformat->layout_typeM=type;
-}
-*/
-
 int
 uinfile_close(UINFORMAT * uinformat)
 {
